<?php
/*
Copyright (C) 2007 Stephane Graber <stgraber@ubuntu.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

function qatracker_main() {
	global $user, $site;
	drupal_add_css('modules/qatracker/qatracker.css');
	drupal_add_css('modules/qawebsite/qawebsite.css');
	drupal_add_js('modules/qawebsite/qawebsite.js');

	$args=func_get_args();
	$site=QAWebsiteSite::getInstance();
	if ($site == Null || $site->title == "Ubuntu QA") { //FIXME: We need a proper implementation of default module
		return qawebsite_sitelist();
	}
	
    $header="<div style=\"color: #000000; font-size: 15px; text-decoration: none;\">".qatracker_getpath($args)."</div>";
	if (count($args) < 1) {
		$args[0]="build";
    }

	switch ($args[0]) {
		case "build":
			if (count($args) > 2)
				$content=qatracker_buildlist($args[1],0,0,$args[2]);
			elseif(count($args) > 1)
				$content=qatracker_buildlist($args[1],0,0,null);
			else
				$content=qatracker_buildlist(null,0,0,null);
		break;
		case "test":
			if (count($args) < 2)
				return drupal_not_found();
			$content=qatracker_testlist($args[1]);
		break;
		case "result":
			if (count($args) < 3)
				return drupal_not_found();
			$content=qatracker_resultlist($args[1],$args[2]);
		break;
		case "dllist":
			$content=qatracker_dllist();
		break;
		case "info":
			if (count($args) < 2)
				return drupal_not_found();
			$content=qatracker_downloadinfo($args[1]);
		break;
		case "report":
			$content=qatracker_report();
		break;
		case "archive":
			if (count($args) < 3)
				return drupal_not_found();
			//$content=qatracker_buildlist($args[1],1,$args[2]);
			$content="<br /><b>Sorry, this feature hasn't been reimplemented yet</b>";
		break;
		case "subscriptions":
			if (count($args) > 1)
				$content=qatracker_adminui_subscription_summary($args[1]);
			else
				$content=qatracker_adminui_subscription_summary();
		break;
		case "testcases":
			$content=qatracker_testcases();
		break;
		case "admin":
			if (user_access($site->adminrole) && count($args) > 1) {
				switch ($args[1]) {
					case "addbuild":
						$content=qatracker_adminui_addbuild();
					break;
					case "edit":
						if (count($args) < 5)
							return drupal_not_found();
						$content=qatracker_adminui_result($args[2],$args[3],$args[4]);
					break;
					case "mailing":
						$content=qatracker_adminui_mailing();
					break;
					case "milestone":
						if (count($args) < 2)
							return drupal_not_found();
						if (count($args) > 3)
							$content=qatracker_adminui_milestone();
						else
							$content=qatracker_adminui_milestone($args[2]);
					break;
                    case "product":
                        $content=qatracker_adminui_product();
                    break;
					case "testcase":
						$content=qatracker_adminui_testcase();
					break;
					default:
						return drupal_not_found();
					break;
				}
			}
			else
				return drupal_access_denied();
		break;
		case "user":
			if (user_access($site->userrole) && count($args) > 1) {
				switch ($args[1]) {
					case "subscription":
						$content=qatracker_subscription();
					break;
					default:
						return drupal_not_found();
					break;
				}
			}
			else
				return drupal_access_denied();
		break;
		default:
			return drupal_not_found();
		break;
	}
	if ($content != null)
		return $header.$content;
	else
		return null;
}

function qatracker_gettitle() {
	$site=QAWebsiteSite::getInstance();
	return $site->title;
}

function qatracker_buildlist($filter,$archive=0,$milestone=0,$status=0) {
	global $base_url, $site;
	$original_filter=$filter;

	// Parse POST data and redirect if needed
	if ($filter != null && $filter != "all")
		$filter=db_result(db_query("SELECT filter FROM qatracker_filter WHERE link='".db_escape_string($filter)."' AND siteid='".$site->id."'"));

	if ($filter == null)
		$filter=$site->getSetting("default_product");
	if ($status != null)
		$status=strtolower($status);

	$content.="<br />";
	if ($site->getSetting("noticeboard")) {
		$content.="<br />".$site->getSetting("noticeboard");
	}
	if ($filter != null && $filter != "all") {
		$filter="^".$filter;
	}
	$content.="<form method=\"post\" action=\"\" id=\"isolist\">";
	if ($archive == 1) {
		$order=" qatracker_build.version ASC,";
		if ($filter != "all") {
			$where="WHERE qatracker_product.title~'".db_escape_string($filter)."' ";
			if ($milestone != "0")
				$where.="AND qatracker_milestone.id='".db_escape_string($milestone)."' ";
		}
		elseif ($milestone != "0")
			$where="WHERE qatracker_milestone.id='".db_escape_string($milestone)."' ";
	}
	else {
		$where="WHERE qatracker_milestone.status!='2' AND (qatracker_build.status='0' OR qatracker_build.status='1') ";
		if ($filter != "all")
			$where.="AND qatracker_product.title~'".db_escape_string($filter)."' ";
	}
	$result=db_query("SELECT
						qatracker_build.id,
						qatracker_build.version,
						qatracker_build.date,
						qatracker_build.status as buildstatus,
						qatracker_milestone.title as milestone,
						qatracker_milestone.status,
						qatracker_product.title as product,
						qatracker_product.type,
						qatracker_product.icon
						FROM qatracker_build
						LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid
						LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid
						$where AND qatracker_product.siteid='".$site->id."'
						ORDER BY milestoneid ASC,$order qatracker_product.title ASC
					");
	$count=0;
	$countv=0;
	$lastVersion="";
	while ($item=db_fetch_object($result)) {
		if ($archive==0)
			$chkclass=htmlentities(str_replace(' ','-',$item->milestone));
		else
			$chkclass=htmlentities(str_replace(' ','-',$item->version));
		if ($item->milestone != $PreviousName) {
			if ($PreviousName)
				$content.="</table>";
			$content.="<h2>".$item->milestone." <small>(".qatracker_getmilestonestatus($item->status).")</small></h2>
			<table class=\"listing\" style=\"margin-bottom:2em\">
				<tr class=\"trheader\">
					<td style=\"width:50%\" align=\"left\"><b>Description</b></td>
					<td style=\"width:10%\" align=\"left\"><b>Mandatory</b></td>
					<td style=\"width:10%\" align=\"left\"><b>Optional</b></td>
					<td style=\"width:20%\" align=\"center\"><b>Bugs</b></td>";
					if (user_access($site->adminrole) && $archive == 0) {
						$content.="<td style=\"width:5%\" align=\"center\"><input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['isolist'], this, '".$chkclass."')\" /></td>";
					}
				$content.="</tr>";
			$PreviousName=$item->milestone;
		}        
		if ($archive == 1 && $item->version != $lastVersion) {
			$id=" class=\"block".$countv."\" style=\"display:none;\"";
			if (user_access($site->adminrole))
				$checkb="<input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['isolist'], this, '".$chkclass."')\" />";
			else
				$checkb="&nbsp;";
			$content.="<tr style=\"background-color:#f6f6f6;\"><td colspan=\"3\" align=\"left\" valign=\"bottom\"><a href=\"javascript:ShowHide('block".$countv."')\">[+] <b>".$item->version." (click to expand)</b></a></td><td style=\"width:5%\" align=\"center\">$checkb</td></tr>";
			$countv++;
			$lastVersion=$item->version;
		}
		if ($item->icon == null)
            $item->icon="info.png";

        $mandatory_build_status = qatracker_getbuildstat($item->id);
        $optional_build_status = qatracker_getbuildstat_optional($item->id);

        $build_status = "";

        if ((ereg("0/", $mandatory_build_status) || ereg("None", $mandatory_build_status)) && (ereg("0/", $optional_build_status) || ereg("None", $optional_build_status))) 
            $build_status = "untested";
        elseif ((ereg("<span style=\"font-weight:bold; color:#006400;", $mandatory_build_status) || ereg("None", $mandatory_build_status)) && (ereg("<span style=\"font-weight:bold; color:#006400;", $optional_build_status) || ereg("None", $optional_build_status)))
            $build_status = "completed";
        else 
            $build_status = "started";

		if ($status == null || $status == "all" || ($status=="untested" && $build_status=="untested") || ($status=="completed" && $build_status=="completed") || ($status=="started" && $build_status=="started") || ($status=="notcompleted" && ($build_status=="started" || $build_status=="untested"))) {
			$content.="
				<tr$id>";
					if ($item->buildstatus != 1 || $archive == 1)
						$content.="<td style=\"width:45%\" align=\"left\" valign=\"bottom\"><a href=\"$base_url/qatracker/info/".$item->id."\"><img src=\"$base_url/modules/qatracker/images/icon/".$item->icon."\" style=\"margin-bottom:-3px;\" alt=\"download info\" /></a> <a href=\"$base_url/qatracker/test/".$item->id."\">".$item->product." (".$item->version.")</a></td>";
					else
						$content.="<td style=\"width:45%\" align=\"left\" valign=\"bottom\"><a href=\"$base_url/qatracker/info/".$item->id."\"><img src=\"$base_url/modules/qatracker/images/icon/".$item->icon."\" style=\"margin-bottom:-3px;\" alt=\"download info\" /></a> <a href=\"$base_url/qatracker/test/".$item->id."\"><span style=\"text-decoration:line-through\">".$item->product." (rebuilding...)</span></a></td>";
					$content.="
					<td style=\"width:10%\" align=\"left\" valign=\"bottom\">".$mandatory_build_status."</td>
					<td style=\"width:10%\" align=\"left\" valign=\"bottom\">".$optional_build_status."</td>
					<td style=\"width:20%\" align=\"center\" valign=\"bottom\">".qatracker_getbugslist($item->id, 0, 0)."</td>";
						if (user_access($site->adminrole)) {
							$content.="<td style=\"width:5%\" align=\"center\" valign=\"bottom\"><input type=\"checkbox\" class=\"".$chkclass."\" value=\"".$item->id."\" name=\"isota_iso[]\" /></td>";
						}
					$content.="
				</tr>";
		}
		$count++;
	}
	if ($count != 0)
		$content.="</table>";
	else
		$content.="<p style=\"text-align:center;font-weight:bold;padding-top:1em;padding-bottom:1em;\">We are not testing at the moment</p>";
	$content.=" <div><br />";
	
	if (user_access($site->adminrole)) {
		$return=qatracker_adminui("build",0,0,$original_filter);
		if ($return == null)
			return null;
		else
			$content.=$return;
	}
	else
		$content.="</div></form>";
	return $content;
}

function qatracker_testlist($buildID) {
	global $base_url, $user, $site;
	$content=qatracker_buildsummary($buildID,$testcaseID)."<br />";
	if (db_result(db_query("SELECT qatracker_milestone.status FROM qatracker_build LEFT JOIN qatracker_milestone on qatracker_milestone.id=qatracker_build.milestoneid WHERE qatracker_build.id='".$buildID."'"))!=2)
		$archive=0;
	else
		$archive=1;

	if (db_result(db_query("SELECT qatracker_build.status FROM qatracker_build WHERE qatracker_build.id='".$buildID."'"))!=0)
		$rebuild=1;
	else
		$rebuild=0;

	if (user_access($site->userrole)) {
		if ($_POST['isot_passnobug'] && is_array($_POST['isota_test'])) {
			foreach ($_POST['isota_test'] as $test) {
				if ($rebuild!=1 && $archive!=1 && db_result(db_query("SELECT count(id) FROM qatracker_result WHERE buildid='".$buildID."' AND testcaseid='".db_escape_string($test)."' AND reporterid='".$user->uid."'"))=="0" && is_numeric($test))
					db_query("INSERT INTO qatracker_result (buildid, testcaseid, reporterid, date, result, status) VALUES ('".$buildID."','".db_escape_string($test)."','".$user->uid."','".date("Y-m-d H:i:s")."','1','0')");
			}
			drupal_set_message("Thank you for your report.", "notice_msg");
			drupal_set_header("Location: $base_url/qatracker/test/$buildID");
			return null;
		}
		if ($_POST['isot_subscribe'] && is_array($_POST['isota_test'])) {
			foreach ($_POST['isota_test'] as $test) {
				if (db_result(db_query("SELECT count(id) FROM qatracker_user_subscription WHERE userid='".$user->uid."' AND testcaseid='".db_escape_string($test)."'"))=="0" && is_numeric($test))
					db_query("INSERT INTO qatracker_user_subscription (userid, testcaseid) VALUES ('".$user->uid."','".db_escape_string($test)."')");
			}
			drupal_set_message("Your subscriptions have been updated.", "notice_msg");
			drupal_set_header("Location: $base_url/qatracker/test/$buildID");
			return null;
		}
		if ($_POST['isot_unsubscribe'] && is_array($_POST['isota_test'])) {
			foreach ($_POST['isota_test'] as $test) {
				db_query("DELETE FROM qatracker_user_subscription WHERE userid='".$user->uid."' AND testcaseid='".db_escape_string($test)."'");
			}
			drupal_set_message("Your subscriptions have been updated.", "notice_msg");
			drupal_set_header("Location: $base_url/qatracker/test/$buildID");
			return null;
		}           
		$content.="<form method=\"post\" action=\"\" id=\"testlist\">";
	}
	if ($site->getSetting("noticeboard")) {
		$content.=$site->getSetting("noticeboard");
	}
	$content.="<h2>Mandatory testcases</h2>";
	$content.="
		<table class=\"listing\" style=\"padding-bottom:2em;\">
			<tr class=\"trheader\">
				<td style=\"width:64%\" align=\"left\"><b>Title</b></td>
				<td style=\"width:7%\" align=\"center\"><b>Reports</b></td>
				<td style=\"width:25%\" align=\"center\"><b>Bugs</b></td>
				<td style=\"width:2%\" align=\"center\">&nbsp;</td>";
					if (user_access($site->userrole)) {
						$content.="<td style=\"width:2%\" align=\"center\"><input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['testlist'], this)\" /></td>";
					}
			$content.="</tr>";
	if ($archive == 0)
		$result=db_query("SELECT
						qatracker_testcase.id, 
						qatracker_testcase.title,
						qatracker_testcase.link
						FROM qatracker_testcase 
						LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_testcase.productid
						WHERE qatracker_build.id='".$buildID."' AND qatracker_testcase.status='0' 
						ORDER BY weight ASC, title ASC");
	else
		$result=db_query("SELECT 
						qatracker_testcase.id, 
						qatracker_testcase.title, 
						qatracker_testcase.link 
						FROM qatracker_testcase 
						LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_testcase.productid 
						LEFT JOIN qatracker_result ON qatracker_result.testcaseid=qatracker_testcase.id AND qatracker_result.buildid=qatracker_build.id 
						WHERE qatracker_build.id='".$buildID."'  
						GROUP BY qatracker_testcase.title, qatracker_testcase.id, qatracker_testcase.link, qatracker_testcase.weight
						HAVING count(qatracker_result.id)>0 
						ORDER BY qatracker_testcase.weight ASC, title ASC;");

	while ($item=db_fetch_object($result)) {
		$content.="
			<tr>";
			if ($item->link)
				$content.="<td style=\"width:64%\" align=\"left\"><a href=\"".$item->link."\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/test.png\" alt=\"test\" /></a> <a href=\"$base_url/qatracker/result/".$buildID."/".$item->id."\">".$item->title."</a></td>";
			else
				$content.="<td style=\"width:64%\" align=\"left\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/test.png\" alt=\"test\" /> <a href=\"$base_url/qatracker/result/".$buildID."/".$item->id."\">".$item->title."</a></td>";
			$content.="<td style=\"width:7%\" align=\"center\">".qatracker_getresultsnumber($buildID,$item->id)."</td>
				<td style=\"width:25%\" align=\"center\">".qatracker_getbugslist($buildID, $item->id, 0)."</td>
				<td style=\"width:2%\" align=\"center\">";
				if (db_result(db_query("SELECT count(id) FROM qatracker_user_subscription WHERE testcaseid='".$item->id."' AND userid='".$user->uid."'")) == 1) {
					$content.="
						<div class=\"balloonleft\">
							<div>
								<b>Subscribers for testcase #".$item->id."</b><br />";
									if (db_result(db_query("SELECT count(id) FROM qatracker_user_subscription WHERE testcaseid='".$item->id."'"))) {
										$content.="<ul>";
										$subscribers=db_query("SELECT userid FROM qatracker_user_subscription WHERE testcaseid='".$item->id."'");
										while ($subscriber=db_fetch_object($subscribers)) {
											$content.="<li>".user_load(array('uid' => $subscriber->userid))->name."</li>";
										}
										$content.="</ul>";
									}
									else {
										$content.="<br />None";
									}
								$content.="
							</div>
							<a rel=\"external\" href=\"\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/sub.png\" alt=\"Subscribed\" /></a>
						</div>";
				}
				else {
					$content.="
						<div class=\"balloonleft\">
							<div>
								<b>Subscribers for testcase #".$item->id."</b><br />";
									if (db_result(db_query("SELECT count(id) FROM qatracker_user_subscription WHERE testcaseid='".$item->id."'"))) {
										$content.="<ul>";
										$subscribers=db_query("SELECT userid FROM qatracker_user_subscription WHERE testcaseid='".$item->id."'");
										while ($subscriber=db_fetch_object($subscribers)) {
											$content.="<li>".user_load(array('uid' => $subscriber->userid))->name."</li>";
										}
										$content.="</ul>";
									}
									else {
										$content.="<br />None";
									}
								$content.="
							</div>
							<a rel=\"external\" href=\"\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/nosub.png\" alt=\"Subscribed\" /></a>
						</div>";
				}
				$content.="</td>";
				if (user_access($site->userrole)) {
					$content.="<td style=\"width:2%\" align=\"center\"><input type=\"checkbox\" name=\"isota_test[]\" value=\"".$item->id."\" /></td>";
				}
			$content.="</tr>";
	}
	$content.="</table><hr />";
	$content.="<h2>Optional testcases</h2>";
	$content.="
		<table class=\"listing\" style=\"padding-bottom:2em;\">
			<tr class=\"trheader\">
				<td style=\"width:64%\" align=\"left\"><b>Title</b></td>
				<td style=\"width:7%\" align=\"center\"><b>Reports</b></td>
				<td style=\"width:25%\" align=\"center\"><b>Bugs</b></td>
				<td style=\"width:2%\" align=\"center\">&nbsp;</td>";
					if (user_access($site->userrole)) {
						$content.="<td style=\"width:2%\" align=\"center\"><input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['testlist'], this)\" /></td>";
					}
			$content.="</tr>";
	if ($archive == 0)
		$result=db_query("SELECT
						qatracker_testcase.id, 
						qatracker_testcase.title,
						qatracker_testcase.link
						FROM qatracker_testcase 
						LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_testcase.productid
						WHERE qatracker_build.id='".$buildID."' AND qatracker_testcase.status='2' 
						ORDER BY weight ASC, title ASC");
	else
		$result=db_query("SELECT 
						qatracker_testcase.id, 
						qatracker_testcase.title, 
						qatracker_testcase.link 
						FROM qatracker_testcase 
						LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_testcase.productid 
						LEFT JOIN qatracker_result ON qatracker_result.testcaseid=qatracker_testcase.id AND qatracker_result.buildid=qatracker_build.id 
						WHERE qatracker_build.id='".$buildID."'  
						GROUP BY qatracker_testcase.title, qatracker_testcase.id, qatracker_testcase.link, qatracker_testcase.weight
						HAVING count(qatracker_result.id)>0 
						ORDER BY qatracker_testcase.weight ASC, title ASC;");

	while ($item=db_fetch_object($result)) {
		$content.="
			<tr>";
			if ($item->link)
				$content.="<td style=\"width:64%\" align=\"left\"><a href=\"".$item->link."\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/test.png\" alt=\"test\" /></a> <a href=\"$base_url/qatracker/result/".$buildID."/".$item->id."\">".$item->title."</a></td>";
			else
				$content.="<td style=\"width:64%\" align=\"left\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/test.png\" alt=\"test\" /> <a href=\"$base_url/qatracker/result/".$buildID."/".$item->id."\">".$item->title."</a></td>";
			$content.="<td style=\"width:7%\" align=\"center\">".qatracker_getresultsnumber($buildID,$item->id)."</td>
				<td style=\"width:25%\" align=\"center\">".qatracker_getbugslist($buildID, $item->id, 0)."</td>
				<td style=\"width:2%\" align=\"center\">";
				if (db_result(db_query("SELECT count(id) FROM qatracker_user_subscription WHERE testcaseid='".$item->id."' AND userid='".$user->uid."'")) == 1) {
					$content.="
						<div class=\"balloonleft\">
							<div>
								<b>Subscribers for testcase #".$item->id."</b><br />";
									if (db_result(db_query("SELECT count(id) FROM qatracker_user_subscription WHERE testcaseid='".$item->id."'"))) {
										$content.="<ul>";
										$subscribers=db_query("SELECT userid FROM qatracker_user_subscription WHERE testcaseid='".$item->id."'");
										while ($subscriber=db_fetch_object($subscribers)) {
											$content.="<li>".user_load(array('uid' => $subscriber->userid))->name."</li>";
										}
										$content.="</ul>";
									}
									else {
										$content.="<br />None";
									}
								$content.="
							</div>
							<a rel=\"external\" href=\"\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/sub.png\" alt=\"Subscribed\" /></a>
						</div>";
				}
				else {
					$content.="
						<div class=\"balloonleft\">
							<div>
								<b>Subscribers for testcase #".$item->id."</b><br />";
									if (db_result(db_query("SELECT count(id) FROM qatracker_user_subscription WHERE testcaseid='".$item->id."'"))) {
										$content.="<ul>";
										$subscribers=db_query("SELECT userid FROM qatracker_user_subscription WHERE testcaseid='".$item->id."'");
										while ($subscriber=db_fetch_object($subscribers)) {
											$content.="<li>".user_load(array('uid' => $subscriber->userid))->name."</li>";
										}
										$content.="</ul>";
									}
									else {
										$content.="<br />None";
									}
								$content.="
							</div>
							<a rel=\"external\" href=\"\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/nosub.png\" alt=\"Subscribed\" /></a>
						</div>";
				}
				$content.="</td>";
				if (user_access($site->userrole)) {
					$content.="<td style=\"width:2%\" align=\"center\"><input type=\"checkbox\" name=\"isota_test[]\" value=\"".$item->id."\" /></td>";
				}
			$content.="</tr>";
	}
	$content.="</table><hr />";
	   
	if (user_access($site->userrole))
		$content.="<table cellpadding=\"5\">";
	if ($rebuild != 1 && $archive != 1 && user_access($site->userrole) && db_result(db_query("SELECT count(id) FROM qatracker_result WHERE buildid='".$buildID."' AND reporterid='".$user->uid."'"))!=db_result(db_query("SELECT count(qatracker_testcase.id) FROM qatracker_testcase LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_testcase.productid WHERE qatracker_build.id='".$buildID."'"))) {
		$content.="<tr><td><input type=\"submit\" name=\"isot_passnobug\" value=\"Passed with no bugs\" /></td><td>Clicking this button will mark all checked tests as passed without any bugs.</td></tr>";
	}
	if (user_access($site->userrole)) {
		$content.="<tr>
					<td><input type=\"submit\" name=\"isot_subscribe\" value=\"Subscribe\" /></td>
					<td rowspan=\"2\">Subscribe/Unsubscribe from selected tests.<br />When subscribed to a test, this one will appear in your \"My subscriptions\" page</td>
				   </tr>
				   <tr>
					<td><input type=\"submit\" name=\"isot_unsubscribe\" value=\"Unsubscribe\" /></td>
				   </tr></table>";
	}
	if (user_access($site->adminrole)) {
		$return=qatracker_adminui("test", $buildID);
		if ($return == null)
			return null;
		else
			$content.=$return;
	}
	elseif (user_access($site->userrole))
		$content.="</form>";
	return $content;
}

function qatracker_resultlist($buildID, $testcaseID) {
	global $user, $base_url, $site;
	$content=qatracker_buildsummary($buildID, $testcaseID);
	$content.="<br />
		<table class=\"listing\">
			<tr class=\"trheader\">
				<td style=\"width:3%\" align=\"left\">&nbsp;</td>
				<td style=\"width:16%\" align=\"left\"><b>Reporter</b></td>
				<td style=\"width:17%\" align=\"left\"><b>Last update</b></td>
				<td style=\"width:17%\" align=\"center\"><b>Bugs</b></td>
				<td style=\"width:42%\" align=\"left\"><b>Comment</b></td>
				<td style=\"width:5%\" align=\"center\">&nbsp;</td>
			</tr>";
	$result=db_query("SELECT id, reporterid, extract(epoch from date) as date, comment, extract(epoch from lastchange) as lastchange, status, result FROM qatracker_result WHERE buildid='".$buildID."' AND testcaseid='".$testcaseID."' ORDER BY id ASC");
	while ($item=db_fetch_object($result)) {
		$show=0;
		if ($item->status == "0") {
			$content.="<tr>";
			$show=1;
		}
		elseif (user_access($site->adminrole)) {
			$content.="<tr style=\"background-color: #bfbfbf\">";
			$show=1;
		}
		
		if ($show == 1) {
			if (!$item->lastchange)
				$item->lastchange=$item->date;
			$item->reporterid=user_load(array('uid' => $item->reporterid))->name;
			if ($item->result == 0)
				$image="<img style=\"margin-top:-3px\" src=\"$base_url/modules/qatracker/images/fail.png\" alt=\"fail\" />";
			elseif ($item->result == 1)
				$image="<img style=\"margin-top:-3px\" src=\"$base_url/modules/qatracker/images/pass.png\" alt=\"pass\" />";
			else
				$image="<img style=\"margin-top:-3px\" src=\"$base_url/modules/qatracker/images/started.png\" alt=\"started\" />";

			if (date("H:i:s",$item->lastchange) == "00:00:00")
				$item->lastchange=date("Y-m-d",$item->lastchange);
			else
				$item->lastchange=gmdate("Y-m-d H:i",$item->lastchange);
			$content.="
					<td style=\"width:3%\" valign=\"top\" align=\"left\">$image</td>
					<td style=\"width:16%\" valign=\"top\" align=\"left\">".$item->reporterid."</td>
					<td style=\"width:17%\" valign=\"top\" align=\"left\">".$item->lastchange."</td>
					<td style=\"width:17%\" valign=\"top\" align=\"center\">".qatracker_getbugslist($buildID,$testcaseID,$item->id)."</td>
					<td style=\"width:42%\" valign=\"top\" align=\"left\">".str_replace("\n","<br />",$item->comment)."</td>
					<td style=\"width:5%\" valign=\"top\" align=\"center\">";
					if (user_access($site->adminrole))
						$content.="<a href=\"$base_url/qatracker/admin/edit/".$buildID."/".$testcaseID."/".$item->id."/\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/edit.png\" alt=\"Edit result\" title=\"Edit this result\" /></a>";
					$content.="</td>
				</tr>";
			$count++;
		}
	}
	$content.="</table><br /><hr />";
	if (db_result(db_query("SELECT qatracker_milestone.status FROM qatracker_build LEFT JOIN qatracker_milestone on qatracker_milestone.id=qatracker_build.milestoneid WHERE qatracker_build.id='".$buildID."'"))!=2)
		$archive=0;
	else
		$archive=1;
	if (db_result(db_query("SELECT qatracker_build.status FROM qatracker_build WHERE qatracker_build.id='".$buildID."'"))!=0)
		$rebuild=1;
	else
		$rebuild=0;
	if (user_access($site->userrole) && $archive!=1 && $rebuild!=1) {
		if (!$count)
			$content.="<p style=\"text-align:center;font-weight:bold;padding-top:1em;padding-bottom:1em;\">No results for that specific test case, maybe would you like to make the first one ?</p>";
		if ( ($_POST['isot_valid']) && (!$_POST['isot_addbug']) ) {
			$result=qatracker_validresult($buildID, $testcaseID);
			if ($result == null)
				return null;
			else
				$content.=$result;
		}
		else {
			$resultID=db_query("SELECT id FROM qatracker_result WHERE buildid='".$buildID."' AND testcaseid='".$testcaseID."' AND reporterid='".$user->uid."' ");
			$resultID=db_result($resultID);
            if ($resultID) {
				$content.="<br /><b>You have already posted a result, the form below lets you update it or delete it:</b>";
				$content.=qatracker_addresult($resultID);
			}
			else
				$content.=qatracker_addresult();
		}
	}
	elseif (user_access($site->userrole))
		$content.="<br />You can't post or edit your result on an archived build or a build being currently rebuilt.";
	else
		$content.="<br />Please register and login to be able to post your own test results.";
	return $content;
}

function qatracker_testcases() {
	global $site;
	$products=array();
	if ($site->id == 1) {
		$products=array(
			"Alternate" => "Alternate",
			"Desktop" => "Desktop",
			"Ubuntu Server" => "Server",
			"DVD" => "DVD",
            "Netbook" => "Netbook",
            "ARM" => "ARM",
			"Netboot" => "Netboot"
		);
	}
	else {
		$productlist=db_query("SELECT title FROM qatracker_product WHERE siteid='".$site->id."'");
		while ($item=db_fetch_object($productlist)) {
			array_push_associative($products, array($item->title => $item->title));
		}
	}
	$content="<br />This page shows how many times each testcase has been tested.<br />Testcases with no results are shown in red, with less than 5 results in orange and with more than 5 results in green.<br />This page doesn't show the result of those tests but only the coverage (if a testcase has 6 failed result it'll be shown in green anyway)";
    foreach ($products as $product => $title) {
		$content.="<h1>$title</h1><table class=\"listing\" style=\"width:auto;\">";
        $tests=db_query("SELECT t.title, count(r.id) FROM qatracker_testcase t 
                                LEFT JOIN qatracker_product p ON p.id = t.productid 
                                LEFT JOIN qatracker_build b ON b.productid = p.id 
                                LEFT JOIN qatracker_milestone m ON b.milestoneid = m.id 
                                LEFT JOIN qatracker_result r ON r.buildid = b.id AND r.testcaseid = t.id 
                                WHERE m.status = 0 
                                AND p.status = '0' AND p.siteid = '".$site->id."' AND p.title~'".$product."'
                                AND b.status = '0'
                                AND t.status = '0'
                                AND (( r.result != '2' AND r.status = '0') OR r.id IS NULL)
                                GROUP BY t.title");
	    $result=false;
		while ($item = db_fetch_object($tests)) {
			if ($item->count == 0)
				$count="<span style=\"color:red; font-weight:bold;\">".$item->count." tests done</span>";
			elseif ($item->count > 5)
				$count="<span style=\"color:green; font-weight:bold;\">".$item->count." tests done</span>";
			else
				$count="<span style=\"color:orange; font-weight:bold;\">".$item->count." tests done</span>";
			$content.="<tr><td style=\"width:25em;\"><b>".$item->title."</b></td><td>$count</td></tr>";
			$result=true;
		}
		if (!$result)
			$content.="<tr><td style=\"width:25em;\" colspan=\"2\"><b>This product is not currently on the tracker.</b></td></tr>";
		$content.="</table>";
    }
	$content.="<h1>Optional Testcases</h1><table class=\"listing\" style=\"width:auto;\">";


    $tests=db_query("SELECT t.title, count(r.id) FROM qatracker_testcase t 
                                LEFT JOIN qatracker_product p ON p.id = t.productid 
                                LEFT JOIN qatracker_build b ON b.productid = p.id 
                                LEFT JOIN qatracker_milestone m ON b.milestoneid = m.id 
                                LEFT JOIN qatracker_result r ON r.buildid = b.id AND r.testcaseid = t.id 
                                WHERE m.status = 0 
                                AND p.status = '0' AND p.siteid = '".$site->id."' 
                                AND b.status = '0' 
                                AND t.status = '2' 
                                AND (( r.result != '2' AND r.status = '0') OR r.id IS NULL) 
                                GROUP BY t.title");
	$result=false;
    while ($item = db_fetch_object($tests)) {
		if ($item->count == 0)
			$count="<span style=\"color:red; font-weight:bold;\">".$item->count." tests done</span>";
		elseif ($item->count > 5)
			$count="<span style=\"color:green; font-weight:bold;\">".$item->count." tests done</span>";
		else
			$count="<span style=\"color:orange; font-weight:bold;\">".$item->count." tests done</span>";
		$content.="<tr><td style=\"width:25em;\"><b>".$item->title."</b></td><td>$count</td></tr>";
		$result=true;
	}
	if (!$result)
		$content.="<tr><td style=\"width:25em;\" colspan=\"2\"><b>This product is not currently on the tracker.</b></td></tr>";
	$content.="</table>";

	return $content;
}

function qatracker_report() {
	global $site;
	$milestones=db_query("SELECT id FROM qatracker_milestone WHERE (status='0' OR status='1') AND siteid='".$site->id."'");
	$content="";
	while ($milestone=db_fetch_object($milestones))
		$content.=qatracker_buildstat($milestone->id);
	if (strlen($content) == 0)
		$content="<br /><b>There is no milestone being currently tested.</b>";
	return $content;
}

function qatracker_buildstat($milestoneID) {
	if (!is_numeric($milestoneID))
		return "";
	$milestone=db_fetch_object(db_query("SELECT id,title,status FROM qatracker_milestone WHERE id='$milestoneID'"));
	$current_bugs=db_query("SELECT
								qatracker_bug.bugnumber, 
								count(qatracker_bug.bugnumber) as dupe 
								FROM qatracker_bug 
								LEFT JOIN qatracker_result ON qatracker_result.id=qatracker_bug.resultid 
								LEFT JOIN qatracker_build ON qatracker_build.id=qatracker_result.buildid 
								LEFT JOIN qatracker_testcase ON qatracker_testcase.id = qatracker_result.testcaseid 
								WHERE (qatracker_build.status='0' OR qatracker_build.status='1') 
								AND qatracker_testcase.status != '2' 
								AND qatracker_build.milestoneid='".$milestoneID."' 
								GROUP BY qatracker_bug.bugnumber 
								ORDER BY dupe DESC,bugnumber ASC");
	$content.="<h1>Stats for ".$milestone->title."</h1>
				<table class=\"listing\" style=\"width:auto;\">
					<tr><td><b>CO</b></td><td>Comments on Launchpad</td></tr>
					<tr><td><b>SU</b></td><td>Subscribers on Launchpad</td></tr>
					<tr><td><b>DU</b></td><td>Duplicates on Launchpad</td></tr>
					<tr><td><b>TT</b></td><td>Number of time the bugs was reported on the tracker</td></tr>
				</table><br />
				<h2>Bugs from mandatory testcases in the current builds:</h2>
					<table class=\"listing\">
						<tr class=\"trheader\">
							<td><b>Bug #</b></td>
							<td><b>Title</b></td>
							<td><b>Affects</b></td>
							<td><b>Status</b></td>
							<td><b>Importance</b></td>
							<td><b>Assignee</b></td>
							<td><b>CO</b></td>
							<td><b>SU</b></td>
							<td><b>DU</b></td>
							<td><b>TT</b></td>
						</tr>";
	while($item = db_fetch_object($current_bugs)) {
		$bug=qawebsite_getbug($item->bugnumber);
		if ($bug) {
			if (strlen($bug->title) > 53)
				$bug->title=substr($bug->title,0,50)."...";
			$content.="
							<tr>
								<td><a href=\"https://bugs.launchpad.net/bugs/".$bug->originalbug."\">".$bug->originalbug."</a></td>
								<td><a href=\"https://bugs.launchpad.net/bugs/".$bug->bugnumber."\">".htmlentities($bug->title)."</a></td>
								<td>".htmlentities($bug->product,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->status,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->importance,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->assignee,ENT_QUOTES,"UTF-8")."</td>
								<td>".$bug->commentscount."</td>
								<td>".$bug->subscriberscount."</td>
								<td>".$bug->duplicatescount."</td>
								<td>".$item->dupe."</td>
							</tr>";
		}
	}
	$content.="</table>";
	$optional_testcases=db_query("SELECT
								qatracker_bug.bugnumber, 
								count(qatracker_bug.bugnumber) as dupe 
								FROM qatracker_bug 
								LEFT JOIN qatracker_result ON qatracker_result.id=qatracker_bug.resultid 
								LEFT JOIN qatracker_build ON qatracker_build.id=qatracker_result.buildid 
								LEFT JOIN qatracker_testcase ON qatracker_testcase.id = qatracker_result.testcaseid 
								WHERE (qatracker_build.status='0' OR qatracker_build.status='1') 
								AND qatracker_testcase.status='2' 
								AND qatracker_build.milestoneid='".$milestoneID."' 
								GROUP BY qatracker_bug.bugnumber 
								ORDER BY dupe DESC,bugnumber ASC");
	$content.="<br />
				<h2>Bugs from optional testcases in the current build:</h2>
					<table class=\"listing\">
						<tr class=\"trheader\">
							<td><b>Bug #</b></td>
							<td><b>Title</b></td>
							<td><b>Affects</b></td>
							<td><b>Status</b></td>
							<td><b>Importance</b></td>
							<td><b>Assignee</b></td>
							<td><b>CO</b></td>
							<td><b>SU</b></td>
							<td><b>DU</b></td>
							<td><b>TT</b></td>
						</tr>";
	while($item = db_fetch_object($optional_testcases)) {
		$bug=qawebsite_getbug($item->bugnumber);
		if ($bug) {
			if (strlen($bug->title) > 53)
				$bug->title=substr($bug->title,0,50)."...";
			$content.="
							<tr>
								<td><a href=\"https://bugs.launchpad.net/bugs/".$bug->originalbug."\">".$bug->originalbug."</a></td>
								<td><a href=\"https://bugs.launchpad.net/bugs/".$bug->bugnumber."\">".htmlentities($bug->title)."</a></td>
								<td>".htmlentities($bug->product,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->status,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->importance,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->assignee,ENT_QUOTES,"UTF-8")."</td>
								<td>".$bug->commentscount."</td>
								<td>".$bug->subscriberscount."</td>
								<td>".$bug->duplicatescount."</td>
								<td>".$item->dupe."</td>
							</tr>";
		}
	}
	$content.="</table>";
	$previous_bugs=db_query("SELECT
								qatracker_bug.bugnumber, 
								count(qatracker_bug.bugnumber) as dupe 
								FROM qatracker_bug 
								LEFT JOIN qatracker_result ON qatracker_result.id=qatracker_bug.resultid 
								LEFT JOIN qatracker_build ON qatracker_build.id=qatracker_result.buildid 
								WHERE (qatracker_build.status!='0' AND qatracker_build.status!='1') AND qatracker_build.milestoneid='".$milestoneID."'
								GROUP BY qatracker_bug.bugnumber 
								ORDER BY dupe DESC,bugnumber ASC");
	$content.="	<br />
				<h2>Bugs for previous builds:</h2>
					<table class=\"listing\">
						<tr class=\"trheader\">
							<td><b>Bug #</b></td>
							<td><b>Title</b></td>
							<td><b>Affects</b></td>
							<td><b>Status</b></td>
							<td><b>Importance</b></td>
							<td><b>Assignee</b></td>
							<td><b>CO</b></td>
							<td><b>SU</b></td>
							<td><b>DU</b></td>
						<td><b>TT</b></td>
					</tr>";
	while($item = db_fetch_object($previous_bugs)) {
		$bug=qawebsite_getbug($item->bugnumber);
		if ($bug) {
			if (strlen($bug->title) > 53)
				$bug->title=substr($bug->title,0,50)."...";
			$content.="
							<tr>
								<td><a href=\"https://bugs.launchpad.net/bugs/".$bug->originalbug."\">".$bug->originalbug."</a></td>
								<td><a href=\"https://bugs.launchpad.net/bugs/".$bug->bugnumber."\">".htmlentities($bug->title)."</a></td>
								<td>".htmlentities($bug->product,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->status,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->importance,ENT_QUOTES,"UTF-8")."</td>
								<td>".htmlentities($bug->assignee,ENT_QUOTES,"UTF-8")."</td>
								<td>".$bug->commentscount."</td>
								<td>".$bug->subscriberscount."</td>
								<td>".$bug->duplicatescount."</td>
								<td>".$item->dupe."</td>
							</tr>";
		}
	}
	$content.="</table><br /><br />";
	return $content;
}

function qatracker_addresult($resultID = 0, $admin = 0) {
	global $user;
	if ($resultID != 0 && !$_POST['isot_addbug'] && !$_POST['isot_valid']) {
		$reportinfo=db_query("SELECT status, reporterid, comment, result FROM qatracker_result WHERE id='".$resultID."'");
		$reportinfo=db_fetch_array($reportinfo);
		$data['isot_comment']=$reportinfo['comment'];
		$data['isot_poster']=$reportinfo['reporterid'];
		$data['isot_status']=$reportinfo['status'];
		$data['isot_result']=$reportinfo['result'];
		$buglist=db_query("SELECT bugnumber, bugimportance FROM qatracker_bug WHERE resultid='".$resultID."' ORDER BY bugnumber ASC");
		$data['isot_bug']=array();
		while ($bug = db_fetch_object($buglist))
			array_push_associative($data['isot_bug'], array($bug->bugnumber => $bug->bugimportance));
		$username=$data['isot_poster'];
	}
    else {
		$data=qatracker_checkpostvalue();
		$username=$user->uid;
	}

	if ($data['isot_result']=='0')
		$selectf="checked=\"checked\"";
	elseif ($data['isot_result']=='1')
		$selectp="checked=\"checked\"";
	else
		$selects="checked=\"checked\"";

	$content="<br />".$data['isot_error']."<br /><form method=\"post\" action=\"\">
		<table cellpadding=\"2\" border=\"0\">
			<tr><td>Username : </td><td><b><i>".user_load(array('uid' => $username))->name."</i></b></td></tr>
			<tr><td style=\"padding-top:1em;\">Result : <span style=\"color:red\">*</span></td><td style=\"padding-top:1em;width:100%\"><input type=\"radio\" name=\"isot_result\" value=\"1\" $selectp />Passed <input type=\"radio\" name=\"isot_result\" value=\"0\" $selectf />Failed <input type=\"radio\" name=\"isot_result\" value=\"2\" $selects />Started</td></tr>
			<tr><td style=\"padding-top:2em;\">Bug ID</td><td style=\"padding-top:2em;width:100%\">Serious</td></tr>";
	$count=0;
	if (count($data['isot_bug'])!=0) {
		foreach ($data['isot_bug'] as $bugnumber => $bugimportance) {
			if ($bugimportance == 1)
				$checked="checked=\"checked\"";
			else
				$checked="";
			$content.="<tr><td><input type=\"text\" size=\"12\" name=\"isot_bugnumber[]\" value=\"$bugnumber\"/></td><td style=\"width:100%\" align=\"left\"><input type=\"checkbox\" name=\"isot_serious[]\" value=\"$count\" $checked/></td></tr>";
			$count++;
		}
    }
	$total=$count;
    if ($count < 3) {
        $total=3;
    }
    elseif ($_POST['isot_addbug'] != "") {
        $total++;
    }
	for ($i=$count;$i<$total;$i++)
		$content.="<tr><td><input type=\"text\" size=\"12\" name=\"isot_bugnumber[]\" /></td><td style=\"width:100%\" align=\"left\"><input type=\"checkbox\" name=\"isot_serious[]\" value=\"$i\"/></td></tr>";
	$content.="
			<tr><td style=\"padding-top:1em;\" colspan=\"2\"><input style=\"width:12em\" type=\"submit\" name=\"isot_addbug\" value=\"More bug entry fields\"/></td></tr>
			<tr><td style=\"padding-top:2em;\" valign=\"top\">Comment : </td><td style=\"padding-top:2em;\"><textarea rows=\"4\" cols=\"50\" name=\"isot_comment\" >".$data['isot_comment']."</textarea></td></tr>";
	$content.="</table>
		<p style=\"padding-top:1em;padding-bottom:1em;\">";
	if ($admin == 1) {
		unset($select);
		if ($data['isot_status'])
			$select="checked=checked";
		$content.="<input type=\"checkbox\" name=\"isot_status\" value=\"1\" $select /> Hide this report<br />";
	}
	if ($resultID != 0) {
        $content.="<input type=\"hidden\" name=\"isot_valid\" value=\"1\"/><input type=\"submit\" name=\"isot_edit\" value=\"Update result\"/>";
        $content.="<br /><br/>Or you can also delete it if you made a mistake...&nbsp;&nbsp;&nbsp;";
		$content.="<input type=\"submit\" name=\"isot_edit\" value=\"Delete result\" />";
	}
	else
		$content.="<input type=\"submit\" style=\"width:12em\" name=\"isot_valid\" value=\"Submit result\"/>";
	$content.="
			</p></form>
			<span style=\"color:red;\">*</span> Required information";
	return $content;
}

function qatracker_validresult($buildID, $testcaseID, $admin = 0, $resultID1 = 0) {
	global $user, $base_url;
    $data=qatracker_checkpostvalue();

	if ($data['isot_error']) {
		return qatracker_addresult($isodID, 0);
	}
	if ($data['isot_edit'] == "Update result" || $data['isot_edit'] == "Delete result") {
		$id=db_query("SELECT id FROM qatracker_result WHERE buildid='".$buildID."' AND testcaseid='".$testcaseID."' AND reporterid='".$user->uid."' ");
		$resultID=db_result($id);
		if ($admin == 1 && $resultID1 != 0) {
			$resultID=$resultID1;
			if ($data['isot_status'] == "1") {
				$status="1";
			}
			else {
				$status="0";
			}
			db_query("UPDATE qatracker_result SET result='".db_escape_string($data['isot_result'])."', lastchange='".date("Y-m-d H:i:s")."', comment='".db_escape_string($data['isot_comment'])."', changedby='".$user->uid."', status='".db_escape_string($status)."' WHERE id='".db_escape_string($resultID)." '");
		}
		else
            if ($data['isot_edit'] == "Update result") {
    			db_query("UPDATE qatracker_result SET result='".db_escape_string($data['isot_result'])."', lastchange='".date("Y-m-d H:i:s")."', comment='".db_escape_string($data['isot_comment'])."' WHERE id='".db_escape_string($resultID)." '");
            }
            else {
		        db_query("DELETE FROM qatracker_result WHERE id='".$resultID."'");
            }
		    db_query("DELETE FROM qatracker_bug WHERE resultid='".$resultID."'");
	}
	else {
		if (db_result(db_query("SELECT count(id) FROM qatracker_result WHERE buildid='".$buildID."' AND testcaseid='".$testcaseID."' AND reporterid='".$user->uid."'"))=="0")
			db_query("INSERT INTO qatracker_result (buildid, testcaseid, reporterid, date, comment, result) VALUES (".$buildID.",".$testcaseID.",".$user->uid.",'".date("Y-m-d H:i:s")."','".db_escape_string($data['isot_comment'])."','".db_escape_string($data['isot_result'])."')");
		$resultID=db_result(db_query("SELECT id FROM qatracker_result WHERE buildid=".$buildID." AND testcaseid=".$testcaseID." AND reporterid=".$user->uid.""));
	}
    $valid=0;
    if ($data['isot_edit'] != "Delete result") {
	    foreach ($data['isot_bug'] as $bugnumber => $bugimportance)
            db_query("INSERT INTO qatracker_bug (resultid, bugnumber, bugimportance) VALUES ('".$resultID."','".db_escape_string($bugnumber)."','".db_escape_string($bugimportance)."')");
    }

    if ($data['isot_edit'] == "Delete result") {
	    drupal_set_message("Your result has been successfully deleted.", "notice_msg");
    }
    else if ($data['isot_edit'] == "Update result") {
	    drupal_set_message("Your report has been updated.", "notice_msg");
    }
    else {
	    drupal_set_message("Thank you for your report.", "notice_msg");
    }
 
	drupal_set_header("Location: $base_url/qatracker/result/$buildID/$testcaseID");
	return null;
}

function qatracker_buildsummary($buildID, $testcaseID=0) {
	$result=db_query("SELECT qatracker_product.title, qatracker_product.icon FROM qatracker_product LEFT JOIN qatracker_build ON qatracker_build.productid=qatracker_product.id WHERE qatracker_build.id='".$buildID."'");
	$build=db_fetch_object($result);
	if ($build->icon == null)
		$build->icon="info.png";
	$content.="<br /><table border=\"0\" width=\"50%\" cellpadding=\"2\">";
	$content.="<tr><td><a style=\"text-decoration:none; color:#000000\" href=\"$base_url/qatracker/info/".$buildID."\"><img src=\"$base_url/modules/qatracker/images/icon/".$build->icon."\" style=\"margin-bottom:-3px;\" alt=\"download info\" /> <b>".$build->title."</b></a></td></tr>";
	if ($testcaseID!=0) {
		$result=db_fetch_array(db_query("SELECT qatracker_testcase.title, qatracker_testcase.link FROM qatracker_testcase WHERE id='".$testcaseID."' "));
	    $content.="<tr><td><h2>Testcase: ".$result['title']."</h2></td></tr>";
    }
	$content.="</table><br />";
	if ($testcaseID && $result['link'])
		$content.="<h3> <a href=\"".$result['link']."\">Read the instructions to perform this test <img src=\"$base_url/modules/qatracker/images/test.png\" style=\"margin-bottom:-3px;\" alt=\"testcase\" /></a></h3><br />";
	return $content;
}

function qatracker_dllist() {
	$query=db_query("SELECT 
		qatracker_build.id 
		FROM qatracker_build
		LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid
		LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid
		WHERE qatracker_milestone.status!='2' AND (qatracker_build.status='0' OR qatracker_build.status='1')
		ORDER BY milestoneid ASC, qatracker_product.title ASC");
	while ($item = db_fetch_object($query)) {
		$data=qatracker_downloadinfo($item->id,1);
		if (is_array($data))
			echo $data['path_iso']." ".str_replace("This file doesn't have a md5","NOMD5",$data['md5'])."\n";
	}
	return False;
}

function qatracker_downloadinfo($buildID, $data_only = 0) {
	global $site;
	//Don't wait 30s to timeout
	ini_set("default_socket_timeout", 1);

	if ($site->id != "1")
		return "<br />Function not implemented yet.";
	$info=db_fetch_object(db_query("SELECT qatracker_build.version as version, qatracker_product.title as product, qatracker_milestone.status as server, qatracker_milestone.title as release FROM qatracker_build LEFT JOIN qatracker_milestone ON qatracker_milestone.id=qatracker_build.milestoneid LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid WHERE qatracker_build.id='".$buildID."'"));
	$url=array();
	$url['release']=split(" ",$info->release);
	$url['release']=strtolower($url['release'][0]);
	
	if ($info->server == "0")
		$url['server']="cdimage.ubuntu.com";
	elseif ($info->server == "1")
		$url['server']="releases.ubuntu.com";
	else
		return "<br />Download information not available.<br />Archived build";

	$url['version']=$info->version;
	$product=str_replace("Ubuntu Studio","ubuntustudio",$info->product);
	$product=str_replace("Kubuntu KDE4","kubuntu-kde4",$product);
	$product=split(" ",$product);

	if (count($product) == 2)
		$url['arch']=$product[1];
	else
		$url['arch']=$product[count($product)-1];

	if (ereg("Upgrade",$info->product))
		return "<br />For upgrades, please use update-manager.";
	elseif (ereg("Desktop",$info->product)) {
        $url['type']="/daily-live/";
        $url['file']=$url['release']."-desktop-".$url['arch'].".iso";
    }
    elseif (ereg("ARM Preinstalled omap3",$info->product)) {
		$url['type']="/ubuntu-netbook/ports/daily-preinstalled/";
		$url['file']=$url['release']."-preinstalled-netbook-armel+omap.img.gz";
	}	
    elseif (ereg("ARM Preinstalled omap4",$info->product)) {
		$url['type']="/ubuntu-netbook/ports/daily-preinstalled/";
		$url['file']=$url['release']."-preinstalled-netbook-armel+omap4.img.gz";
	}	
	elseif (ereg("Alternate",$info->product)) {
		$url['type']="/daily/";
		$url['file']=$url['release']."-alternate-".$url['arch'].".iso";
	}
	elseif (ereg("JeOS",$info->product)) {
		$url['type']="/jeos/daily/";
		$url['file']=$url['release']."-jeos-".$url['arch'].".iso";
	}
	elseif (ereg("DVD",$info->product)) {
		$url['type']="/dvd/";
		$url['file']=$url['release']."-dvd-".$url['arch'].".iso";
	}
	elseif (ereg("add-on",$info->product)) {
		$url['type']="/daily/";
		$url['file']=$url['release']."-addon-".$url['arch'].".iso";
	}
	elseif (ereg("Server",$info->product)) {
		$url['type']="/ubuntu-server/daily/";
		$url['file']=$url['release']."-server-".$url['arch'].".iso";
	}
	elseif (ereg("Netboot",$info->product)) {
		$url['path']="http://archive.ubuntu.com/ubuntu/dists/".$url['release']."/main/installer-".$url['arch']."/current/images/";
	}
	elseif (ereg("Ubuntu Netbook",$info->product)) {
		if (ereg("armel",$info->product)) {
			$url['type']="/ubuntu-netbook/ports/daily-live/";
			$url['file']=$url['release']."-netbook-".$url['arch'].".img";
		}
		else {
			$url['type']="/ubuntu-netbook/daily-live/";
			$url['file']=$url['release']."-netbook-i386.iso";
		}
	}
	elseif (ereg("Kubuntu Netbook",$info->product)) {
		$url['type']="/kubuntu-netbook/daily-live/";
		$url['file']=$url['release']."-netbook-i386.iso";
	}
	elseif (ereg("Moblin",$info->product)) {
		$url['type']="/ubuntu-moblin-remix/daily-live/";
		$url['file']=$url['release']."-moblin-remix-i386.iso";
	}
	else
		return "<br />Unknown build type or title";

	$url['distro']=ereg_replace("^ubuntu$","",strtolower($product[0]));
	if ($url['distro'] != "")
		$url['distro']="/".$url['distro'];

	#Kubuntu Netbook is not in the kubuntu folder
	if (ereg("Kubuntu Netbook",$info->product)) 
		$url['distro']= "";

	if ($url['path'])
		$url['path_iso']=$url['path'];
	else {
		$url['path_iso']="http://".$url['server'].$url['distro'].$url['type'].$url['version']."/".$url['file'];
		$url['path_md5']="http://".$url['server'].$url['distro'].$url['type'].$url['version']."/MD5SUMS";
	}

	$header=@get_headers($url['path_iso']);
	if (ereg('404 Not Found',$header[0]))
		return "<br />This build wasn't found on ".$url['path_iso']." (may no longer exists)";
	
	if ($url['path_md5'])
		$header=@get_headers($url['path_md5']);
	else
		$header=null;

	if ($header && !ereg('404 Not Found',$header[0])) {
		$handle_md5=@fopen($url['path_md5'],"r");
		if ($handle_md5) {
			while (!feof($handle_md5)) {
				$line=fgets($handle_md5);
				if (ereg($url['file'],$line))
					$md5=$line;
			}
			fclose($handle_md5);
		}
		$url['md5']=ereg_replace(" .*","",$md5);
	}
	else
		$url['md5']="This file doesn't have a md5";
	
	$content=qatracker_buildsummary($buildID)."<br /><br />";
	$content.="
		<table class=\"listing\" style=\"width:auto\">
			<tr><td><b>Direct download : </b></td><td><a href=\"".$url['path_iso']."\">".$url['path_iso']."</a></td></tr>
            <tr><td><b>MD5Sum : </b></td><td>".$url['md5']."</td></tr>
			<tr><td><b>zsync : </b></td><td>zsync ".$url['path_iso'].".zsync</td></tr>
			<tr><td><b>rsync : </b></td><td>rsync -tzhhP rsync://".str_replace("http://","",str_replace(".com/",".com/cdimage/",$url['path_iso']))." .</td></tr>
		</table>
		<br />You can read more about zsync and rsync on the wiki : <a href=\"https://help.ubuntu.com/community/ZsyncCdImage\">community/ZsyncCdImage</a>";
	if ($data_only == 1)
		return $url;
	else
		return "<br />".$content;
}

function qatracker_subscription() {
	global $user, $base_url, $site;
	if (user_access($site->userrole)) {
		if ($_POST['isot_passnobug'] && is_array($_POST['isota_test'])) {
			foreach ($_POST['isota_test'] as $test) {
				$info=explode(";",$test);
				$test=$info[0];
				$buildID=$info[1];
				if (db_result(db_query("SELECT qatracker_milestone.status FROM qatracker_build LEFT JOIN qatracker_milestone on qatracker_milestone.id=qatracker_build.milestoneid WHERE qatracker_build.id='".$buildID."'"))!=2)
					$archive=0;
				else
					$archive=1;
				if (db_result(db_query("SELECT qatracker_build.status FROM qatracker_build WHERE qatracker_build.id='".$buildID."'"))!=0)
					$rebuild=1;
				else
					$rebuild=0;
				if ($rebuild!=1 && $archive!=1 && db_result(db_query("SELECT count(id) FROM qatracker_result WHERE buildid='".$buildID."' AND testcaseid='".db_escape_string($test)."' AND reporterid='".$user->uid."'"))=="0" && is_numeric($test))
					db_query("INSERT INTO qatracker_result (buildid, testcaseid, reporterid, date, result, status) VALUES ('".$buildID."','".db_escape_string($test)."','".$user->uid."','".date("Y-m-d H:i:s")."','1','0')");
			}
			drupal_set_message("Thank you for your report.", "notice_msg");
			drupal_set_header("Location: $base_url/qatracker/user/subscription");
			return null;
		}
		if ($_POST['isot_unsubscribe'] && is_array($_POST['isota_test'])) {
			foreach ($_POST['isota_test'] as $test) {
				$info=explode(";",$test);
				$buildID=$info[1];
				$test=$info[0];
				db_query("DELETE FROM qatracker_user_subscription WHERE userid='".$user->uid."' AND testcaseid='".db_escape_string($test)."'");
			}
			drupal_set_message("Your subscriptions have been updated.", "notice_msg");
			drupal_set_header("Location: $base_url/qatracker/user/subscription");
			return null;
		}           
		$content.="<form method=\"post\" action=\"\" id=\"testlist\">";
	}
	$content.="<p><br /><b>You are currently subscribed to the followings tests :</b><br /><br /></p>";
	$content.="
		<table class=\"listing\" style=\"padding-bottom:2em;\">
			<tr class=\"trheader\">
				<td align=\"left\"><b>Build</b></td>
				<td align=\"left\"><b>Version</b></td>
				<td align=\"left\"><b>Test</b></td>
				<td style=\"display:table-cell;display:none\" class=\"hidethis\" align=\"center\"><b>Reports</b></td>
				<td style=\"display:table-cell;display:none\" class=\"hidethis\" align=\"center\"><b>Bugs</b></td>
				<td style=\"width:0;\" align=\"center\">&nbsp;</td>
				<td style=\"width:0;\" align=\"center\">&nbsp;</td>
				<td style=\"width:0;\" align=\"center\">&nbsp;</td>
			</tr>";
	$subscribedtest=db_query("SELECT qatracker_build.version, qatracker_product.title as product, qatracker_product.icon, qatracker_milestone.title as milestone, qatracker_testcase.title as testcase, qatracker_build.id as buildid, qatracker_testcase.id as testcaseid, qatracker_product.id as productid FROM qatracker_build LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid LEFT JOIN qatracker_testcase ON qatracker_testcase.productid=qatracker_build.productid LEFT JOIN qatracker_user_subscription ON qatracker_user_subscription.testcaseid=qatracker_testcase.id LEFT JOIN qatracker_milestone on qatracker_milestone.id=qatracker_build.milestoneid WHERE qatracker_user_subscription.userid='".$user->uid."' AND qatracker_build.status='0' AND qatracker_milestone.status!='2' AND qatracker_testcase.status='0' ORDER by qatracker_milestone.id ASC, qatracker_product.title ASC, qatracker_build.version ASC, qatracker_testcase.id ASC");
	$lastbuild=0;
	$countv=0;
	while ($item = db_fetch_object($subscribedtest)) {
		if ($item->icon == null)
			$item->icon="info.png";
		if ($lastbuild != $item->buildid) {
			$countv++;
			$total=db_result(db_query("SELECT count(qatracker_user_subscription.id) FROM qatracker_user_subscription LEFT JOIN qatracker_testcase ON qatracker_testcase.id=qatracker_user_subscription.testcaseid WHERE qatracker_testcase.productid='".$item->productid."' AND qatracker_testcase.status='0' AND qatracker_user_subscription.userid='".$user->uid."'"));
			$done=db_result(db_query("SELECT count(qatracker_user_subscription.id) FROM qatracker_result LEFT JOIN qatracker_user_subscription ON qatracker_user_subscription.testcaseid=qatracker_result.testcaseid LEFT JOIN qatracker_testcase ON qatracker_testcase.id=qatracker_result.testcaseid WHERE qatracker_result.buildid='".$item->buildid."' AND qatracker_result.reporterid='".$user->uid."' AND qatracker_user_subscription.userid='".$user->uid."' AND qatracker_testcase.status='0'"));
			$content.="<tr style=\"background-color:#f6f6f6;\">
						<td align=\"left\"><a href=\"javascript:ShowHide('block".$countv."')\">[+] <b>".$item->product." (click to expand)</b></a></td>
						<td style=\"display:table-cell\" align=\"left\" class=\"block".$countv."\">".$item->version." (".$item->milestone.")</td>
						<td style=\"display:table-cell;\" class=\"block".$countv."\"><b>$done/$total</b></td>
						<td colspan=\"7\">&nbsp;</td></tr>";
			$lastbuild=$item->buildid;
		}
		$content.="
			<tr class=\"block".$countv."\" style=\"display:none\">
				<td align=\"left\"><a href=\"$base_url/qatracker/info/".$item->buildid."\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/icon/".$item->icon."\" alt=\"build\" /></a> <a href=\"$base_url/qatracker/test/".$item->buildid."\">".$item->product."</a></td>
				<td align=\"left\">".$item->version." (".$item->milestone.")</td>
				<td align=\"left\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/test.png\" alt=\"test\" /> <a href=\"$base_url/qatracker/result/".$item->buildid."/".$item->testcaseid."\">".$item->testcase."</a></td>
				<td align=\"center\">".qatracker_getresultsnumber($item->buildid,$item->testcaseid)."</td>
				<td align=\"center\">".qatracker_getbugslist($item->buildid, $item->testcaseid, 0)."</td>";
				$item->result=db_result(db_query("SELECT result FROM qatracker_result WHERE buildid='".$item->buildid."' AND testcaseid='".$item->testcaseid."' AND reporterid='".$user->uid."'"));
				if ($item->result == "0" && db_result(db_query("SELECT count(id) FROM qatracker_result WHERE buildid='".$item->buildid."' AND testcaseid='".$item->testcaseid."' AND reporterid='".$user->uid."'")) != "0")
					$content.="<td align=\"center\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/fail.png\" alt=\"spacer\" /></td>";
				elseif ($item->result == "1")
					$content.="<td align=\"center\"><img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/pass.png\" alt=\"spacer\" /></td>";
				else
					$content.="<td align=\"center\">&nbsp;</td>";
				$content.="<td align=\"center\">
					<div class=\"balloonleft\">
						<a rel=\"external\" href=\"\">
							<img style=\"margin-bottom:-3px;\" src=\"$base_url/modules/qatracker/images/sub.png\" alt=\"Subscribed\" />
						</a>
						<div>
							<b>Subscribers for testcase #".$item->testcaseid."</b><br />
							<ul>";
								$subscribers=db_query("SELECT userid FROM qatracker_user_subscription WHERE testcaseid='".$item->testcaseid."'");
								while ($subscriber=db_fetch_object($subscribers)) {
									$content.="<li>".user_load(array('uid' => $subscriber->userid))->name."</li>";
								}
							$content.="</ul>
						</div>
					</div>
				</td>
				<td align=\"center\"><input type=\"checkbox\" name=\"isota_test[]\" value=\"".$item->testcaseid.";".$item->buildid."\" /></td>
			</tr>";
	}
	$content.=" </table><div><hr />
				<br /><input type=\"submit\" name=\"isot_passnobug\" value=\"Passed with no bugs\" /> Clicking this button will mark all checked tests as passed without any bugs.<br />
				<br /><input type=\"submit\" name=\"isot_unsubscribe\" value=\"Unsubscribe\" /> Unsubscribe from the selected tests.
				</div></form>";
	return $content;
}

function qatracker_adminui($page, $buildID = 0, $testcaseID = 0, $distro = "All") {
	global $site;
	if (user_access($site->adminrole)) {
		$content="<div style=\"padding-top:2em;\"><hr /><h2>Administration</h2></div>";
		$return="";
		switch ($page) {
			case "build" :
				$return=qatracker_adminui_build($distro);
			break;
			case "test" :
				$return=qatracker_adminui_test($buildID);
			break;
		}
	}
	if ($return == null)
		return null;
	else
		return $content.=$return;
}

function qatracker_adminui_build($distro) {
	global $base_url, $site;
	if ($_POST['isota_selisodisable'] && is_array($_POST['isota_iso'])) {
		foreach ($_POST['isota_iso'] as $iso) {
			db_query("UPDATE qatracker_build SET status='1' WHERE id='".db_escape_string($iso)."' ");
		}
		drupal_set_message(count($_POST['isota_iso'])." builds have been disabled.", "notice_msg");
		drupal_set_header("Location: $base_url/qatracker/build/$distro");
		return null;
	}
	elseif ($_POST['isota_selisoenable'] && is_array($_POST['isota_iso'])) {
		foreach ($_POST['isota_iso'] as $iso) {
			db_query("UPDATE qatracker_build SET status='0' WHERE id='".db_escape_string($iso)."' ");
		}
		drupal_set_message(count($_POST['isota_iso'])." builds have been enabled.", "notice_msg");
		drupal_set_header("Location: $base_url/qatracker/build/$distro");
		return null;
	}
	elseif ($_POST['isota_selisoremove'] && is_array($_POST['isota_iso'])) {
		foreach ($_POST['isota_iso'] as $iso) {
			db_query("UPDATE qatracker_build SET status='2' WHERE id='".db_escape_string($iso)."' ");
		}
		drupal_set_message(count($_POST['isota_iso'])." builds have been removed.", "notice_msg");
		drupal_set_header("Location: $base_url/qatracker/build/$distro");
		return null;
	}
	elseif ($_POST['isota_changenotice']  && $_POST['isota_notice']) {
		$site->setSetting("noticeboard",$_POST['isota_notice']);
		drupal_set_message("Notice message has successfully been updated.", "notice_msg");
		drupal_set_header("Location: $base_url/qatracker/build/$distro");
		return null;
	}
	else {
		$content="
				<table width=\"0\" border=\"0\" cellpadding=\"5\">
					<tr><td align=\"left\"><input type=\"submit\" name=\"isota_selisodisable\" value=\"Disable selection\" /></td><td align=\"left\">Make the selection inactive</td></tr>
					<tr><td align=\"left\"><input type=\"submit\" name=\"isota_selisoenable\" value=\"Re-enable selection\" /></td><td align=\"left\">Make the selection active</td></tr>
					<tr><td align=\"left\"><input type=\"submit\" name=\"isota_selisoremove\" value=\"Remove selection\" /></td><td align=\"left\">Remove the selection, should only be used to remove a build which has been added by mistake. (Use Disable to mark one for rebuild)</td></tr>
				</table>";
		$notice=$site->getSetting("noticeboard");
		$content.="<br /><p><b>Change notice message</b><br /><textarea name=\"isota_notice\" rows=\"13\" style=\"width:70em;height:10em;\" cols=\"30\">".htmlentities($notice)."</textarea><br /><br /><input type=\"submit\" name=\"isota_changenotice\" value=\"Change notice\" /></p></div></form>";
	}
	return $content;
}

function qatracker_adminui_test($buildID) {
	$show=1;
	$content="";
	if ($_POST['isota_subpeople'] && $_POST['isota_subpeoplelist'] != "" && is_array($_POST['isota_test'])) {
		$usernames=$_POST['isota_subpeoplelist'];
		$list=explode(",",str_replace(" ","",$_POST['isota_subpeoplelist']));
		$listuid=array();
		$show=0;
		foreach ($list as $people) {
			if (user_load(array('name' => $people))->uid != "")
				array_push($listuid,user_load(array('name' => $people))->uid);
			else
				$show=1;
		}
		if ($show == 0) {
			foreach ($listuid as $peopleid) {
				foreach ($_POST['isota_test'] as $test) {
					if (db_result(db_query("SELECT count(id) FROM qatracker_user_subscription WHERE userid='".$peopleid."' AND testcaseid='".db_escape_string($test)."'"))=="0" && is_numeric($test))
						db_query("INSERT INTO qatracker_user_subscription (userid, testcaseid) VALUES ('".$peopleid."','".db_escape_string($test)."')");
				}
			}
			drupal_set_message("Updated subscriptions for ".count($listuid)." users.","notice_msg");
			drupal_set_header("Location: $base_url/qatracker/test/$buildID");
			return null;
		}
	}
	elseif ($_POST['isota_unsubpeople'] && $_POST['isota_subpeoplelist'] != "" && is_array($_POST['isota_test'])) {
		$usernames=$_POST['isota_subpeoplelist'];
		$list=explode(",",str_replace(" ","",$_POST['isota_subpeoplelist']));
		$listuid=array();
		$show=0;
		foreach ($list as $people) {
			if (user_load(array('name' => $people))->uid != "")
				array_push($listuid,user_load(array('name' => $people))->uid);
			else
				$show=1;
		}
		if ($show == 0) {
			foreach ($listuid as $peopleid) {
				foreach ($_POST['isota_test'] as $test) {
					db_query("DELETE FROM qatracker_user_subscription WHERE userid='".$peopleid."' and testcaseid='".db_escape_string($test)."'");
				}
			}
			drupal_set_message("Updated subscriptions for ".count($listuid)." users.","notice_msg");
			drupal_set_header("Location: $base_url/qatracker/test/$buildID");
			return null;
		}
	}
	if ($show != 0) {
		$content.="<div>";
		if ($_POST['isota_subpeople'] && $show == 1)
			$content.="<span style=\"color:red\">Error: Please check that you have at least selected one test above and that the specified usernames are valid and comma separated.</span><br />";
		$content.="<input type=\"text\" name=\"isota_subpeoplelist\" value=\"$usernames\"/> <input type=\"submit\" name=\"isota_subpeople\" value=\"Subscribe people\" /> <input type=\"submit\" name=\"isota_unsubpeople\" value=\"Unsubscribe people\" /> (Un)Subscribe those usernames to the selected tests (comma separated)</div></form>";
	}
	return $content;
}

function qatracker_adminui_milestone($milestoneid = 0) {
	global $base_url, $site;
	if ($_POST['isota_releaserenametext'] != "" && $_POST['isota_release']) {
		if (count($_POST['isota_release']) == 1) {
			db_query("UPDATE qatracker_milestone SET title='".htmlentities($_POST['isota_releaserenametext'])."' WHERE qatracker_milestone.id='".db_escape_string($_POST['isota_release'][0])."'");
			drupal_set_message("Milestone successfully renamed.","notice_msg");
		}
		drupal_set_header("Location: $base_url/qatracker/admin/milestone");
		return null;
	}
	if ($milestoneid!=0 && $_POST['isota_iso'] && $_POST['isota_releasemoveiso'] && is_numeric($_POST['isota_releasemilestone'])) {
		foreach ($_POST['isota_iso'] as $iso)
			db_query("UPDATE qatracker_build SET milestoneid='".db_escape_string($_POST['isota_releasemilestone'])."' WHERE qatracker_build.id='".db_escape_string($iso)."'");
		drupal_set_message(count($_POST['isota_iso'])." builds successfully moved.","notice_msg");
		drupal_set_header("Location: $base_url/qatracker/admin/milestone/$milestoneid");
		return null;
	}
	if ($milestoneid!=0 && $_POST['isota_iso'] && $_POST['isota_releaseversion'] && $_POST['isota_releaseversionchange']) {
		foreach ($_POST['isota_iso'] as $iso)
			db_query("UPDATE qatracker_build SET version='".htmlentities($_POST['isota_releaseversion'])."' WHERE qatracker_build.id='".db_escape_string($iso)."'");
		drupal_set_message(count($_POST['isota_iso'])." builds successfully updated.");
		drupal_set_header("Location: $base_url/qatracker/admin/milestone/$milestoneid");
	}
	elseif ($milestoneid!=0) {
		$releasename=db_result(db_query("SELECT qatracker_milestone.title FROM qatracker_milestone WHERE qatracker_milestone.id='".db_escape_string($milestoneid)."'"));
		$content.="<h1>Milestone \"$releasename\"</h1><br /><b>Builds for this milestone</b><br /><br />";
		$tests=db_query("SELECT qatracker_build.id, qatracker_build.version, qatracker_product.title FROM qatracker_build LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid WHERE qatracker_build.milestoneid='".db_escape_string($milestoneid)."' ORDER BY qatracker_build.version DESC, qatracker_product.title ASC");
		$previousversion="0";
		$content.="<form method=\"post\" id=\"isolist\" action=\"\"><table class=\"listing\" style=\"margin-bottom:1em;width:auto;\">";
		$count=0;
		$countv=0;
		while ($item = db_fetch_object($tests)) {
			if ($previousversion != $item->version) {
				$countv++;
				$content.="
						<tr style=\"background-color:#f6f6f6;\">
							<td><a href=\"javascript:ShowHide('block".$countv."')\">[+] <b>".$item->version."</b></a></td>
							<td><input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['isolist'], this, '".$item->version."')\" /></td>
						</tr>";
				$previousversion=$item->version;
			}
			$content.="<tr class=\"block".$countv."\" style=\"display:none\"><td>".$item->title."</td><td><input type=\"checkbox\" class=\"".$item->version."\" value=\"".$item->id."\" name=\"isota_iso[]\" /></td></tr>";
			$count++;
		}
		$content.="</table><p>";
			if ($count != 0) {
				$content.="<select name=\"isota_releasemilestone\">";
				$milestone_list=db_query("SELECT qatracker_milestone.id, qatracker_milestone.title FROM qatracker_milestone ORDER BY qatracker_milestone.title");
				while ($item=db_fetch_object($milestone_list)) {
					if ($item->id == $site->getSetting('default_milestone'))
						$content.="<option value=\"".$item->id."\" selected=\"selected\">".$item->title."</option>";
					else
						$content.="<option value=\"".$item->id."\">".$item->title."</option>";
				}
				$content.="</select> 
							<input type=\"submit\" name=\"isota_releasemoveiso\" value=\"Move selected to milestone\"/><br /><br />
							<input type=\"text\" name=\"isota_releaseversion\" /> <input type=\"submit\" name=\"isota_releaseversionchange\" value=\"Change build number of selection\"/><br /><br />";
			}
			else
				$content.="No build to add<br /><br />";
			$content.="
			</p></form>";
	}
	elseif ($_POST['isota_releasename'] && $_POST['isota_releaseadd']) {
		$total=db_query("SELECT count(qatracker_milestone.id) from qatracker_milestone WHERE qatracker_milestone.title='".htmlentities($_POST['isota_releasename'])."'");
		$total=db_result($total);
		if ($total == 0) {
			db_query("INSERT INTO qatracker_milestone (title, status, siteid) VALUES ('".htmlentities($_POST['isota_releasename'])."','0','".$site->id."')");
			drupal_set_message("Milestone successfully added.","notice_msg");
		}
		drupal_set_header("Location: $base_url/qatracker/admin/milestone");
		return null;
	}
	elseif (is_numeric($_POST['isota_releasedefault']) && $_POST['isota_releasesetdefault']) {
		$site->setSetting("default_milestone",$_POST['isota_releasedefault']);
		drupal_set_message("Default milestone successfully changed.","notice_msg");
		drupal_set_header("Location: $base_url/qatracker/admin/milestone");
		return null;
	}
	elseif ($_POST['isota_release'] && $_POST['isota_releasedelete']) {
		$count=0;
		foreach ($_POST['isota_release'] as $release) {
			$iso=db_query("SELECT count(id) from qatracker_build WHERE milestoneid='".db_escape_string($release)."'");
			$iso=db_result($iso);
			if ($iso == 0 && $release != $site->getSetting(default_milestone)) {
				db_query("UPDATE qatracker_milestone SET status='3' WHERE qatracker_milestone.id='".db_escape_string($release)."'");
				$count++;
			}
		}
		drupal_set_message($count." milestones removed.","notice_msg");
		drupal_set_header("Location: $base_url/qatracker/admin/milestone");
		return null;
	}
	elseif ($_POST['isota_release'] && $_POST['isota_releasestatusupdate'] && is_numeric($_POST['isota_releasestatus'])) {
		foreach ($_POST['isota_release'] as $release) {
			db_query("UPDATE qatracker_milestone SET status='".db_escape_string($_POST['isota_releasestatus'])."' WHERE qatracker_milestone.id='".db_escape_string($release)."'");
		}
		drupal_set_message(count($_POST['isota_release'])." milestones updated.","notice_msg");
		drupal_set_header("Location: $base_url/qatracker/admin/milestone");
	}
	else {
		$content="<br /><form id=\"releaselist\" method=\"post\" action=\"\">
			<table class=\"listing\" style=\"margin-bottom:1em;width:auto;\">
				<tr class=\"trheader\">
					<td><b>Status</b></td>
					<td><b>Milestone</b></td>
					<td><input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['releaselist'], this)\" /></td>
				</tr>
		";
		$tests=db_query("SELECT DISTINCT qatracker_milestone.title, qatracker_milestone.status, qatracker_milestone.id, count(qatracker_build.id) as countiso FROM qatracker_milestone LEFT JOIN qatracker_build ON qatracker_build.milestoneid=qatracker_milestone.id WHERE qatracker_milestone.siteid='".$site->id."' AND qatracker_milestone.status!='3' GROUP BY qatracker_milestone.title, qatracker_milestone.status, qatracker_milestone.id ORDER BY qatracker_milestone.status DESC, qatracker_milestone.title ASC");
		$count=0;
		$listrelease=array();
		while ($item = db_fetch_object($tests)) {
			$content.="<tr><td>".qatracker_getmilestonestatus($item->status)."</td><td><a href=\"$base_url/qatracker/admin/milestone/".$item->id."\">".$item->title." (".$item->countiso." builds)</a></td><td><input type=\"checkbox\" name=\"isota_release[]\" value=\"".$item->id."\"/></td></tr>";
			$count++;
			array_push_associative($listrelease, array($item->id => $item->title));
		}
		$content.="</table><p>
			<select name=\"isota_releasestatus\">
				<option value=\"0\">".qatracker_getmilestonestatus(0)."</option>
				<option value=\"1\">".qatracker_getmilestonestatus(1)."</option>
				<option value=\"2\">".qatracker_getmilestonestatus(2)."</option>                           
			</select>
			<input type=\"submit\" name=\"isota_releasestatusupdate\" value=\"Update selected status\" /><br /><br />
			<select name=\"isota_releasedefault\">";
		foreach ($listrelease as $id => $title) {
			if ($id == $site->getSetting('default_milestone'))
				$content.="<option value=\"$id\" selected=\"selected\">$title</option>";
			else
				$content.="<option value=\"$id\">$title</option>";
		}
		$content.="
			</select>
			<input type=\"submit\" name=\"isota_releasesetdefault\" value=\"Set default milestone\" /><br /><br />
			<input type=\"text\" name=\"isota_releasename\" /> <input type=\"submit\" name=\"isota_releaseadd\" value=\"Add milestone\" /><br /><br />
			<input type=\"text\" name=\"isota_releaserenametext\" /> <input type=\"submit\" name=\"isota_releaserename\" value=\"Rename selected\" /><br /><br />
			<input type=\"submit\" name=\"isota_releasedelete\" value=\"Delete selected\" /><br /><br />
			<b>Note</b> : You can delete a milestone only if it doesn't have any build and it isn't the current default milestone.<br />
			You can only rename one milestone at once, selecting more than one will do nothing.
		</p></form>";
	}
	return $content;
}

function qatracker_adminui_result($buildID, $testcaseID, $resultID) {
	if ( ($_POST['isot_valid']) && (!$_POST['isot_addbug'])) {
		$data=qatracker_checkpostvalue();
		if ($data['isot_error']) {
			$content.=qatracker_addresult($resultID, 1);
		}
		else {
			$content.=qatracker_validresult($buildID, $testcaseID, 1, $resultID);
		}
	}
	else {
		$content=qatracker_addresult($resultID, 1);
	}
	return $content;
}

function qatracker_adminui_addbuild() {
	global $base_url, $site, $user;
	if ($_POST['isota_addisoadv'] && $_POST['isota_version'] && $_POST['isota_release'] && is_array($_POST['isota_case'])) {
		$buildlist=array();
		foreach ($_POST['isota_case'] as $number) {
			$test=db_result(db_query("SELECT count(id) FROM qatracker_build WHERE version='".db_escape_string($_POST['isota_version'])."' AND productid='".db_escape_string($number)."' AND milestoneid='".db_escape_string($_POST['isota_release'])."'"));
			if ($test == 0) {
				db_query("UPDATE qatracker_build SET status='3' WHERE productid='".db_escape_string($number)."' AND milestoneid='".db_escape_string($_POST['isota_release'])."'");
				db_query("INSERT INTO qatracker_build (version, productid, date, status, milestoneid, userid) VALUES ('".db_escape_string(trim($_POST['isota_version']))."', '".db_escape_string($number)."', '".date("Y-m-d H:i:s")."', '0', '".db_escape_string($_POST['isota_release'])."', '".$user->uid."')");
				array_push($buildlist,db_result(db_query("SELECT id FROM qatracker_build WHERE version='".db_escape_string($_POST['isota_version'])."' AND productid='".db_escape_string($number)."' AND milestoneid='".db_escape_string($_POST['isota_release'])."' AND userid='".$user->uid."'")));
			}
		}
		$email_count=qatracker_event_newbuild($buildlist);
		drupal_set_message(count($buildlist)." builds have correctly been added.<br />".$email_count." notification e-mails have been sent.", "notice_msg");
		drupal_set_header("Location: $base_url/qatracker/");
		return null;
	}
	else {
		$content="<br /><form id=\"testlist\" method=\"post\" action=\"\">";
		$tests=db_query("SELECT id, title FROM qatracker_product WHERE siteid='".$site->id."' AND status='0' ORDER BY title ASC");
		$olddistro="";
		while ($item = db_fetch_object($tests)) {
			$distribution=explode(" ",$item->title);
			$distribution=$distribution[0];
			if ($olddistro != $distribution) {
				if ($olddistro != "")
					$content.="</table>";
				$content.="<table class=\"listing\" style=\"margin-bottom:1em;width:25em;\">
				<tr class=\"trheader\" style=\"width:0\">
					<td><b>Product</b></td>
					<td><input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['testlist'], this, '".$distribution."')\" /></td>
				</tr>";
				$olddistro=$distribution;
			}
			$content.="<tr><td>".$item->title."</td><td><input type=\"checkbox\" class=\"".$distribution."\" name=\"isota_case[]\" value=\"".$item->id."\" /></td></tr>";
		}
		$content.="</table><p>
			<input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['testlist'], this)\" />Check all<br /><br />
			<b>Version number</b> <input type=\"text\" name=\"isota_version\" /><br /><br />
			<select name=\"isota_release\">
			";
		$listrelease=db_query("SELECT qatracker_milestone.id, qatracker_milestone.title FROM qatracker_milestone WHERE qatracker_milestone.siteid='".$site->id."' ORDER BY qatracker_milestone.status ASC, qatracker_milestone.title ASC");
		while ($release = db_fetch_object($listrelease)) {
			if ($release->id == $site->getSetting('default_milestone'))
				$content.="<option value=\"".$release->id."\" selected=\"selected\">".$release->title."</option>";
			else
				$content.="<option value=\"".$release->id."\">".$release->title."</option>";
		}
		$content.="</select> <input type=\"submit\" name=\"isota_addisoadv\" value=\"Add build(s)\" />
		</p></form>";
		return $content;
	}
}

function qatracker_adminui_testcase() {
    global $base_url, $site, $user;
    if ($_POST['isota_addtestcase'] && is_array($_POST['isota_case'])) {
        $tc_title = trim(db_escape_string($_POST['isota_testcasetitle']));
        $tc_url = trim(db_escape_string($_POST['isota_testcaseurl']));
        if ($tc_title == "") {
            drupal_set_message("The title of the testcase cannot be empty.");
        }
        else { 
	    	foreach ($_POST['isota_case'] as $number) {
		    	$test=db_result(db_query("SELECT count(id) FROM qatracker_testcase WHERE title='".$tc_title."' AND productid='".db_escape_string($number)."'"));
			    if ($test == 0) {
				    db_query("INSERT INTO qatracker_testcase (productid, title, link) VALUES ('".db_escape_string($number)."', '".$tc_title."', '".$tc_url."')");
			    }
	    	}
            drupal_set_message("The testcases have correctly been added.<br />", "notice_msg");
        }
		    drupal_set_header("Location: $base_url/qatracker/admin/testcase");
            return null;
	}
	else {
		$content="<br /><form id=\"testlist\" method=\"post\" action=\"\">";
		$tests=db_query("SELECT id, title FROM qatracker_product WHERE siteid='".$site->id."' AND status='0' ORDER BY title ASC");
		$olddistro="";
		while ($item = db_fetch_object($tests)) {
			$distribution=explode(" ",$item->title);
			$distribution=$distribution[0];
			if ($olddistro != $distribution) {
				if ($olddistro != "")
					$content.="</table>";
				$content.="<table class=\"listing\" style=\"margin-bottom:1em;width:25em;\">
				<tr class=\"trheader\" style=\"width:0\">
					<td><b>".$distribution."</b></td>
					<td><input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['testlist'], this, '".$distribution."')\" /></td>
				</tr>";
				$olddistro=$distribution;
			}
			$content.="<tr><td>".$item->title."</td><td><input type=\"checkbox\" class=\"".$distribution."\" name=\"isota_case[]\" value=\"".$item->id."\" /></td></tr>";
		}
		$content.="</table><p>
			<input type=\"checkbox\" name=\"checkall\" onclick=\"checkAll(document.forms['testlist'], this)\" />Check all<br /><br />
			<b>Testcase title</b> <input type=\"text\" name=\"isota_testcasetitle\" /><br /><br />
			<b>Testcase URL</b> <input type=\"text\" name=\"isota_testcaseurl\" /><br /><br />
			";
		$content.="<input type=\"submit\" name=\"isota_addtestcase\" value=\"Add testcase\" />
		</p></form>";
		return $content;
	}
}

function qatracker_adminui_product() {
	global $base_url, $site, $user;
    if ($_POST['isota_addproduct'] && $_POST['isota_product']) {
        $product = trim(db_escape_string(($_POST['isota_product'])));
        if ($product == "") {
            drupal_set_message("The product name cannot be blank.");
        }
        else {
            $test=db_result(db_query("SELECT count(id) from qatracker_product WHERE title='".$product."' AND siteid = '".$site->id."'"));
            if ($test > 0) {
                drupal_set_message("That product already exists. Please, choose another name.");
            }
            else {
                db_query("INSERT INTO qatracker_product (siteid, title, type) VALUES ('".$site->id."','".$product."', 0)");
                drupal_set_message("The product was successfully added.");
            }
        }

		drupal_set_header("Location: $base_url/qatracker/admin/product");
		return null;
	}
    else {
		$content="<br /><form id=\"productlist\" method=\"post\" action=\"\">";
		$tests=db_query("SELECT id, title FROM qatracker_product WHERE siteid='".$site->id."' AND status='0' ORDER BY title ASC");
		$olddistro="";
		while ($item = db_fetch_object($tests)) {
			$distribution=explode(" ",$item->title);
			$distribution=$distribution[0];
			if ($olddistro != $distribution) {
				if ($olddistro != "")
					$content.="</table>";
				$content.="<table class=\"listing\" style=\"margin-bottom:1em;width:25em;\">
				<tr class=\"trheader\" style=\"width:0\">
					<td><b>".$distribution."</b></td>
				</tr>";
				$olddistro=$distribution;
			}
			$content.="<tr><td>".$item->title."</td></tr>";
		}
		$content.="</table><p>
			<b>Product name</b> <input type=\"text\" name=\"isota_product\" /><br /><br />
            <input type=\"submit\" name=\"isota_addproduct\" value=\"Add product\" />
		    </p></form>";
		return $content;
	}
}



function qatracker_adminui_report($milestone, $type) {
	if ($milestone == 0)
		$content.="You must specify a milestone (using the drop-down menu at the bottom of the main page)";
	elseif ($type == "full")
		$content=qatracker_adminui_showreport_milestone($milestone);
	else
		$content=qatracker_adminui_showreport_lastrelease($milestone);
	return $content;
}

function qatracker_adminui_showreport_milestone($milestone) {
	$builds=db_query("
		SELECT
		qatracker_result.id as resultid,
		qatracker_build.version,
		qatracker_product.id as productid,
		qatracker_product.title,
		qatracker_result.reporterid
		FROM qatracker_build
		LEFT JOIN qatracker_result ON qatracker_result.buildid=qatracker_build.id
		LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid
		WHERE qatracker_build.milestoneid='".db_escape_string($milestone)."'
		ORDER BY qatracker_product.title ASC, qatracker_result.reporterid ASC, qatracker_build.version ASC, qatracker_result.id ASC
		;");
	$rcount=0;
	$acount=0;
	$pbuild="";
	$milestone_name=db_result(db_query("SELECT title FROM qatracker_milestone WHERE qatracker_milestone.id='".db_escape_string($milestone)."'"));
	$content="<h1>$milestone_name</h1><br /><br /><table width=\"90%\"><tr><td><b>Product</b></td><td><b>Test (Testers)</b></td><td><b>Bug (Critical)</b></td><td><b>Testcase (Total)</b></td></tr>";
	while ($item=db_fetch_object($builds)) {
		if ($lastproduct != $item->productid) {
			if ($lastproduct) {
				$content.="<tr><td>".$pbuild[0]."</td><td>".$acount." (".$rcount.")</td><td>".qatracker_getbugreportstat($pbuild[1])."</td><td>".qatracker_getbuildreportstat($pbuild[1])."</td></tr>";
				unset($lastproduct);
				unset($lastreporter);
				$rcount=0;
				$acount=0;
			}
			$pbuild=array($item->title, $item->productid);
			$lastproduct=$item->productid;
		}
		if ($item->reporterid != $lastreporter) {
			$rcount++;
			$lastreporter=$item->reporterid;
		}
		if ($item->resultid)
			$acount++;
	}
	if (is_array($pbuild))
		$content.="<tr><td>".$pbuild[0]."</td><td>".$acount." (".$rcount.")</td><td>".qatracker_getbugreportstat($pbuild[1])."</td><td>".qatracker_getbuildreportstat($pbuild[1])."</td></tr>";
	$content.="</table>";
	return $content;
}

function qatracker_adminui_showreport_lastrelease($milestone) {
	$builds=db_query("
		SELECT
		qatracker_result.id as resultid,
		qatracker_build.id as buildid,
		qatracker_build.version,
		qatracker_product.id as productid,
		qatracker_product.title,
		qatracker_result.reporterid
		FROM qatracker_build
		LEFT JOIN qatracker_result ON qatracker_result.buildid=qatracker_build.id
		LEFT JOIN qatracker_product ON qatracker_product.id=qatracker_build.productid
		WHERE qatracker_build.milestoneid='".db_escape_string($milestone)."' AND qatracker_build.status!='3'
		ORDER BY qatracker_product.title ASC, qatracker_result.reporterid ASC, qatracker_build.version ASC, qatracker_result.id ASC
		;");
	$rcount=0;
	$acount=0;
	$pbuild="";
	$milestone_name=db_result(db_query("SELECT title FROM qatracker_milestone WHERE qatracker_milestone.id='".db_escape_string($milestone)."'"));
	$content="<h1>$milestone_name</h1><br /><br /><table width=\"90%\"><tr><td><b>Product</b></td><td><b>Test (Testers)</b></td><td><b>Bug (Critical)</b></td><td><b>Testcase (Total)</b></td></tr>";
	while ($item=db_fetch_object($builds)) {
		if ($lastproduct != $item->productid) {
			if ($lastproduct) {
				$content.="<tr><td>".$pbuild[0]."</td><td>".$acount." (".$rcount.")</td><td>".qatracker_getbugreportstat($pbuild[1],$pbuild[2])."</td><td>".qatracker_getbuildreportstat($pbuild[1],$pbuild[2])."</td></tr>";
				unset($lastproduct);
				unset($lastreporter);
				$rcount=0;
				$acount=0;
			}
			$pbuild=array($item->title,$item->productid,$item->buildid);
			$lastproduct=$item->productid;
		}
		if ($item->reporterid != $lastreporter) {
			$rcount++;
			$lastreporter=$item->reporterid;
		}
		if ($item->resultid)
			$acount++;
	}
	if (is_array($pbuild))
		$content.="<tr><td>".$pbuild[0]."</td><td>".$acount." (".$rcount.")</td><td>".qatracker_getbugreportstat($pbuild[1],$pbuild[2])."</td><td>".qatracker_getbuildreportstat($pbuild[1],$pbuild[2])."</td></tr>";
	$content.="</table>";
	return $content;
}

function qatracker_adminui_mailing() {
	global $user, $base_url;
	if ($_POST['isota_mailconfirm'] && $_POST['isota_mailsubmit'] && is_numeric($_POST['isota_mailtarget']) && $_POST['isota_mailsubject'] != "" && $_POST['isota_mailbody'] != "") {
		$recipient=array();
		switch ($_POST['isota_mailtarget']) {
			case 1:
				$result=db_query("SELECT DISTINCT reporterid FROM qatracker_result");
				while ($item = db_fetch_object($result))
					array_push($recipient,user_load(array('uid' => $item->reporterid))->mail);
			break;
			default:
				$recipient=array($user->mail);
			break;
		}
		$subject=urldecode($_POST['isota_mailsubject']);
		$body=urldecode($_POST['isota_mailbody']);
		foreach($recipient as $to)
			mail($to,$subject,$body,"From: QA Testing Tracker <qatracker@stgraber.org>\nContent-Type: text/plain; charset=utf-8");
		drupal_set_message("Your e-mail has been sent.", "notice_msg");
		drupal_set_header("Location: $base_url/qatracker/");
		return null;
	}
	elseif ($_POST['isota_mailsubmit'] && is_numeric($_POST['isota_mailtarget']) && $_POST['isota_mailsubject'] != "" && $_POST['isota_mailbody'] != "") {
		switch ($_POST['isota_mailtarget']) {
			case 1:
				$count=db_result(db_query("SELECT COUNT(DISTINCT reporterid) FROM qatracker_result")); 
			break;
			default;
				$count=1;
			break;
		}
		$content.="
			<br /><b>You are about to send the following mail to ".$count." recipients</b><br /><br />
			<b>Subject :</b> ".$_POST['isota_mailsubject']."<br />
			<b>Body :</b><br /><pre>".$_POST['isota_mailbody']."</pre><br /><br />
			Check that the above is correct and then confirm.<br /><br />
			<form action=\"\" method=\"post\"><div>
			<input type=\"hidden\" name=\"isota_mailbody\" value=\"".urlencode($_POST['isota_mailbody'])."\" />
			<input type=\"hidden\" name=\"isota_mailsubject\" value=\"".urlencode($_POST['isota_mailsubject'])."\" />
			<input type=\"hidden\" name=\"isota_mailtarget\" value=\"".$_POST['isota_mailtarget']."\" />
			<input type=\"hidden\" name=\"isota_mailsubmit\" value=\"".$_POST['isota_mailsubmit']."\" />
			<input type=\"submit\" name=\"isota_mailconfirm\" value=\"Send\" />
			</div></form>
		";
	}
	else {
		$content.="<br /><form method=\"post\" action=\"\"><div>
			<b>Subject :</b><br />
			<input type=\"text\" name=\"isota_mailsubject\" style=\"width:40em;background-color:#f2f2f2;\" /><br /><br />
			<b>Body :</b><br />
			<textarea cols=\"74\" rows=\"28\" name=\"isota_mailbody\"></textarea><br /><br />
			<select name=\"isota_mailtarget\" style=\"width:40em;\">
				<option value=\"1\">Send to everyone who contributed to Ubuntu QA</option>
				<option value=\"2\">Send to myself (for testing purpose)</option>
			</select><br /><br />
			<input type=\"submit\" name=\"isota_mailsubmit\" value=\"Send\"/>
			</div></form>";
	}
	return $content;
}

function qatracker_adminui_subscription_summary($view = "normal") {
	global $user, $site;
	$content="<br />";
	switch ($view) {
		case "supported":
			$query=db_query("SELECT
				qatracker_product.title as product, 
				users.name, 
				qatracker_testcase.title as testcase 
				FROM qatracker_testcase 
				LEFT JOIN qatracker_product on qatracker_product.id=qatracker_testcase.productid 
				LEFT JOIN qatracker_user_subscription ON qatracker_user_subscription.testcaseid=qatracker_testcase.id
				LEFT JOIN users ON users.uid=qatracker_user_subscription.userid
				WHERE qatracker_product.siteid='".$site->id."' AND qatracker_testcase.status='0' AND qatracker_product.status='0' AND qatracker_product.title!~'Ubuntu Studio' AND qatracker_product.title!~'Xubuntu'
				ORDER BY qatracker_product.title ASC, qatracker_testcase.weight ASC, qatracker_testcase.title ASC, users.name ASC");
		break;
		default:
			$query=db_query("SELECT
				qatracker_product.title as product, 
				users.name, 
				qatracker_testcase.title as testcase 
				FROM qatracker_testcase 
				LEFT JOIN qatracker_product on qatracker_product.id=qatracker_testcase.productid 
				LEFT JOIN qatracker_user_subscription ON qatracker_user_subscription.testcaseid=qatracker_testcase.id
				LEFT JOIN users ON users.uid=qatracker_user_subscription.userid
				WHERE qatracker_product.siteid='".$site->id."' AND qatracker_testcase.status='0' AND qatracker_product.status='0'
				ORDER BY qatracker_product.title ASC, qatracker_testcase.weight ASC, qatracker_testcase.title ASC, users.name ASC");
		break;
	}
	$lastproduct=0;
	$lasttestcase=0;
	$subscriptions=array();
	$product=array();
	$subscribers=array();
	$users=array();
	while ($item=db_fetch_object($query)) {
		if ($lastproduct != $item->product && $lastproduct!="0") {
			array_push_associative($product,array($lasttestcase => $subscribers));
			array_push_associative($subscriptions,array($lastproduct => $product));
			$product=array();
			$subscribers=array();
			$lasttestcase=$item->testcase;
		}
		if ($lasttestcase != $item->testcase && $lasttestcase!="0") {
			array_push_associative($product,array($lasttestcase => $subscribers));
			$subscribers=array();
		}
		if ($item->name) {
			array_push($subscribers,$item->name);
			if (!isset($users[$item->name]))
				if (user_access($site->developerrole,user_load(array("name" => $item->name))))
					$users[$item->name]="1";
				else
					$users[$item->name]="0";
		}
		$lastproduct=$item->product;
		$lasttestcase=$item->testcase;
	}
	array_push_associative($product,array($lasttestcase => $subscribers));
	array_push_associative($subscriptions,array($lastproduct => $product));
	foreach ($subscriptions as $product => $testcases) {
		$content.="<table class=\"listing\">";
		$content.="<tr class=\"trheader\"><td colspan=\"2\" align=\"center\"><b><a href=\"/qatracker/build/$product/\">$product</a></b></td></tr>";
		foreach ($testcases as $testcase => $subscribers) {
			$content.="<tr><td align=\"left\" style=\"width:50%;\">$testcase</td><td style=\"width:50%;\" align=\"right\">";
			$first=True;
			foreach ($subscribers as $username) {
				if (!$first) {
					$content.=", ";
				}
				if ($users[$username] == "1")
					$content.="<b>".$username."</b>";
				else
					$content.=$username;
				$first=False;
			}
			$content.="</td></tr>";
		}
		$content.="</table><br />";
	}	
	return $content;
}

function qatracker_actionblock() {
	$site=new QAWebsiteSite();
	$menu=array();
	if (arg(1))
		$page="/qatracker/".arg(1);
	else
		$page="/qatracker/build";
	if (arg(2))
		$filter="/".arg(2);
	else
		$filter="/".strtolower(db_result(db_query("SELECT link FROM qatracker_filter WHERE filter='".db_escape_string($site->getSetting("default_product"))."'")));
	if (arg(3))
		$status="/".arg(3);
	else
		$status="/all";

	//Page related action menu
	$menu['Actions']=array();
	if (arg(1) == "archive")
		$menu['Actions']['Normal view']="/qatracker/build".$filter.$status;
	elseif (arg(1) == "build" || arg(1) == "")
		$menu['Actions']['Archive view']="/qatracker/archive".$filter.$status;
	else
		$menu['Actions']['Build list']="/qatracker/";
	$menu['Actions']['Subscription summary']="/qatracker/subscriptions";
	$menu['Actions']['Testcase coverage']="/qatracker/testcases";
	$menu['Actions']['Show report']="/qatracker/report";
	if (user_access("Registered User")) {
		$menu['Actions']['My subscriptions']="/qatracker/user/subscription";
	}

	//Administration pages
	if (user_access($site->adminrole)) {
		$menu['Administration']=array();
		$menu['Administration']['Add a build set']="/qatracker/admin/addbuild";
		$menu['Administration']['Write an email']="/qatracker/admin/mailing";
		$menu['Administration']['Manage products']="/qatracker/admin/product";
		$menu['Administration']['Manage testcases']="/qatracker/admin/testcase";
		$menu['Administration']['Manage milestones']="/qatracker/admin/milestone";
	}

	if ($page == "/qatracker/build" || $page == "/qatracker/archive") {
		//Status menu
		$menu['Status']=array();
		if (arg(3) == "all" || !arg(3))
			$menu['Status']['<b>All</b>']=$page.$filter."/all";
		else
			$menu['Status']['All']=$page.$filter."/all";
	    if (arg(3) == "notcompleted")
			$menu['Status']['<b>Not Completed</b>']=$page.$filter."/notcompleted";
		else
			$menu['Status']['Not Completed']=$page.$filter."/notcompleted";
	    if (arg(3) == "untested")
			$menu['Status']['<b>Untested</b>']=$page.$filter."/untested";
		else
			$menu['Status']['Untested']=$page.$filter."/untested";
		if (arg(3) == "started")
			$menu['Status']['<b>Started</b>']=$page.$filter."/started";
		else
			$menu['Status']['Started']=$page.$filter."/started";
		if (arg(3) == "completed")
			$menu['Status']['<b>Completed</b>']=$page.$filter."/completed";
		else
			$menu['Status']['Completed']=$page.$filter."/completed";

		//Filtering menu
		$menu['Filter']=array();
		$category=db_query("SELECT title,link FROM qatracker_filter WHERE siteid='".$site->id."' ORDER BY weight ASC");
		while ($item=db_fetch_object($category)) {
			if ("/".$item->link == $filter)
				$item->title="<b>".$item->title."</b>";
			array_push_associative($menu['Filter'],array($item->title => $page."/".$item->link.$status));
		}
	}

	return $menu;
}
?>
